import unittest
from unittest.mock import patch, Mock, MagicMock
import json

from pygeai.cli.commands.evaluation import (
    show_help,
    list_datasets,
    create_dataset,
    get_dataset,
    update_dataset,
    delete_dataset,
    create_dataset_row,
    list_dataset_rows,
    get_dataset_row,
    update_dataset_row,
    delete_dataset_row,
    create_dataset_row_expected_source,
    list_dataset_row_expected_sources,
    get_dataset_row_expected_source,
    update_dataset_row_expected_source,
    delete_dataset_row_expected_source,
    create_dataset_row_filter_variable,
    list_dataset_row_filter_variables,
    get_dataset_row_filter_variable,
    update_dataset_row_filter_variable,
    delete_dataset_row_filter_variable,
    update_dataset_rows_file,
    list_evaluation_plans,
    create_evaluation_plan,
    get_evaluation_plan,
    update_evaluation_plan,
    delete_evaluation_plan,
    list_evaluation_plan_system_metrics,
    add_evaluation_plan_system_metric,
    get_evaluation_plan_system_metric,
    update_evaluation_plan_system_metric,
    delete_evaluation_plan_system_metric,
    list_system_metrics,
    get_system_metric,
    execute_evaluation_plan,
    list_evaluation_results,
    get_evaluation_result
)
from pygeai.core.common.exceptions import MissingRequirementException, WrongArgumentError


class TestEvaluationCommands(unittest.TestCase):
    """
    python -m unittest pygeai.tests.cli.commands.test_evaluation.TestEvaluationCommands
    """

    def setUp(self):
        self.eval_client_patcher = patch('pygeai.evaluation.clients.EvaluationClient.__init__', return_value=None)
        self.mock_eval_client_init = self.eval_client_patcher.start()
        
    def tearDown(self):
        self.eval_client_patcher.stop()

    def test_show_help(self):
        with patch('pygeai.core.utils.console.Console.write_stdout') as mock_stdout:
            show_help()
            mock_stdout.assert_called_once()

    # Dataset Commands Tests
    def test_list_datasets(self):
        with patch('pygeai.evaluation.dataset.clients.EvaluationDatasetClient.list_datasets', return_value="Datasets list") as mock_list:
            with patch('pygeai.core.utils.console.Console.write_stdout') as mock_stdout:
                list_datasets()
                mock_list.assert_called_once()
                mock_stdout.assert_called_once_with("Feedback detail: \nDatasets list")

    def test_create_dataset_with_name(self):
        option_list = [
            (Mock(spec=['name'], name="dataset_name"), "Test Dataset"),
            (Mock(spec=['name'], name="dataset_description"), "Test Description"),
            (Mock(spec=['name'], name="dataset_type"), "T"),
            (Mock(spec=['name'], name="dataset_active"), "1")
        ]
        for opt, _ in option_list:
            opt.name = opt._mock_name

        with patch('pygeai.evaluation.dataset.clients.EvaluationDatasetClient.create_dataset', return_value="Created dataset") as mock_create:
            with patch('pygeai.core.utils.console.Console.write_stdout') as mock_stdout:
                create_dataset(option_list)
                mock_create.assert_called_once()
                mock_stdout.assert_called_once_with("New dataset detail: \nCreated dataset")

    def test_create_dataset_with_file(self):
        option_list = [
            (Mock(spec=['name'], name="dataset_file"), "/path/to/file.json")
        ]
        option_list[0][0].name = "dataset_file"

        with patch('pygeai.evaluation.dataset.clients.EvaluationDatasetClient.create_dataset_from_file', return_value="Created from file") as mock_create:
            with patch('pygeai.core.utils.console.Console.write_stdout') as mock_stdout:
                create_dataset(option_list)
                mock_create.assert_called_once_with(file_path="/path/to/file.json")
                mock_stdout.assert_called_once_with("New dataset detail: \nCreated from file")

    def test_create_dataset_with_rows_dict(self):
        row_data = {
            "dataSetRowExpectedAnswer": "answer",
            "dataSetRowContextDocument": "context",
            "dataSetRowInput": "input"
        }
        option_list = [
            (Mock(spec=['name'], name="dataset_name"), "Test Dataset"),
            (Mock(spec=['name'], name="row"), json.dumps(row_data))
        ]
        for opt, _ in option_list:
            opt.name = opt._mock_name

        with patch('pygeai.evaluation.dataset.clients.EvaluationDatasetClient.create_dataset', return_value="Created dataset") as mock_create:
            with patch('pygeai.core.utils.console.Console.write_stdout') as mock_stdout:
                create_dataset(option_list)
                mock_create.assert_called_once()

    def test_create_dataset_with_rows_list(self):
        row_data = [
            {
                "dataSetRowExpectedAnswer": "answer",
                "dataSetRowContextDocument": "context",
                "dataSetRowInput": "input"
            }
        ]
        option_list = [
            (Mock(spec=['name'], name="dataset_name"), "Test Dataset"),
            (Mock(spec=['name'], name="row"), json.dumps(row_data))
        ]
        for opt, _ in option_list:
            opt.name = opt._mock_name

        with patch('pygeai.evaluation.dataset.clients.EvaluationDatasetClient.create_dataset', return_value="Created dataset") as mock_create:
            with patch('pygeai.core.utils.console.Console.write_stdout') as mock_stdout:
                create_dataset(option_list)
                mock_create.assert_called_once()

    def test_create_dataset_invalid_row_json(self):
        option_list = [
            (Mock(spec=['name'], name="dataset_name"), "Test Dataset"),
            (Mock(spec=['name'], name="row"), "invalid json")
        ]
        for opt, _ in option_list:
            opt.name = opt._mock_name

        with self.assertRaises(WrongArgumentError):
            create_dataset(option_list)

    def test_create_dataset_missing_name(self):
        option_list = []
        with self.assertRaises(MissingRequirementException) as context:
            create_dataset(option_list)
        self.assertEqual(str(context.exception), "Cannot create dataset without specifying dataset name")

    def test_get_dataset_success(self):
        option_list = [
            (Mock(spec=['name'], name="dataset_id"), "dataset-123")
        ]
        option_list[0][0].name = "dataset_id"

        with patch('pygeai.evaluation.dataset.clients.EvaluationDatasetClient.get_dataset', return_value="Dataset detail") as mock_get:
            with patch('pygeai.core.utils.console.Console.write_stdout') as mock_stdout:
                get_dataset(option_list)
                mock_get.assert_called_once_with(dataset_id="dataset-123")
                mock_stdout.assert_called_once_with("Dataset detail: \nDataset detail")

    def test_get_dataset_missing_id(self):
        option_list = []
        with self.assertRaises(MissingRequirementException) as context:
            get_dataset(option_list)
        self.assertEqual(str(context.exception), "Cannot retrieve dataset without specifying id.")

    def test_update_dataset_success(self):
        option_list = [
            (Mock(spec=['name'], name="dataset_id"), "dataset-123"),
            (Mock(spec=['name'], name="dataset_name"), "Updated Name"),
            (Mock(spec=['name'], name="dataset_description"), "Updated Description")
        ]
        for opt, _ in option_list:
            opt.name = opt._mock_name

        with patch('pygeai.evaluation.dataset.clients.EvaluationDatasetClient.update_dataset', return_value="Updated dataset") as mock_update:
            with patch('pygeai.core.utils.console.Console.write_stdout') as mock_stdout:
                update_dataset(option_list)
                mock_update.assert_called_once()
                mock_stdout.assert_called_once_with("Updated dataset detail: \nUpdated dataset")

    def test_update_dataset_missing_id(self):
        option_list = [
            (Mock(spec=['name'], name="dataset_name"), "Updated Name")
        ]
        option_list[0][0].name = "dataset_name"

        with self.assertRaises(MissingRequirementException) as context:
            update_dataset(option_list)
        self.assertEqual(str(context.exception), "Cannot update dataset without specifying id.")

    def test_delete_dataset_success(self):
        option_list = [
            (Mock(spec=['name'], name="dataset_id"), "dataset-123")
        ]
        option_list[0][0].name = "dataset_id"

        with patch('pygeai.evaluation.dataset.clients.EvaluationDatasetClient.delete_dataset', return_value="Deleted") as mock_delete:
            with patch('pygeai.core.utils.console.Console.write_stdout') as mock_stdout:
                delete_dataset(option_list)
                mock_delete.assert_called_once_with(dataset_id="dataset-123")
                mock_stdout.assert_called_once_with("Deleted dataset detail: \nDeleted")

    def test_delete_dataset_missing_id(self):
        option_list = []
        with self.assertRaises(MissingRequirementException) as context:
            delete_dataset(option_list)
        self.assertEqual(str(context.exception), "Cannot delete dataset without specifying id.")

    # Dataset Row Commands Tests
    def test_create_dataset_row_success(self):
        row_data = {
            "dataSetRowExpectedAnswer": "answer",
            "dataSetRowContextDocument": "context",
            "dataSetRowInput": "input"
        }
        option_list = [
            (Mock(spec=['name'], name="dataset_id"), "dataset-123"),
            (Mock(spec=['name'], name="row"), json.dumps(row_data))
        ]
        for opt, _ in option_list:
            opt.name = opt._mock_name

        with patch('pygeai.evaluation.dataset.clients.EvaluationDatasetClient.create_dataset_row', return_value="Created row") as mock_create:
            with patch('pygeai.core.utils.console.Console.write_stdout') as mock_stdout:
                create_dataset_row(option_list)
                mock_create.assert_called_once()

    def test_create_dataset_row_invalid_json(self):
        option_list = [
            (Mock(spec=['name'], name="dataset_id"), "dataset-123"),
            (Mock(spec=['name'], name="row"), "invalid json")
        ]
        for opt, _ in option_list:
            opt.name = opt._mock_name

        with self.assertRaises(WrongArgumentError):
            create_dataset_row(option_list)

    def test_create_dataset_row_missing_id(self):
        row_data = {"dataSetRowExpectedAnswer": "answer"}
        option_list = [
            (Mock(spec=['name'], name="row"), json.dumps(row_data))
        ]
        option_list[0][0].name = "row"

        with self.assertRaises(MissingRequirementException) as context:
            create_dataset_row(option_list)
        self.assertEqual(str(context.exception), "Cannot create dataset row without specifying id.")

    def test_list_dataset_rows_success(self):
        option_list = [
            (Mock(spec=['name'], name="dataset_id"), "dataset-123")
        ]
        option_list[0][0].name = "dataset_id"

        with patch('pygeai.evaluation.dataset.clients.EvaluationDatasetClient.list_dataset_rows', return_value="Rows list") as mock_list:
            with patch('pygeai.core.utils.console.Console.write_stdout') as mock_stdout:
                list_dataset_rows(option_list)
                mock_list.assert_called_once_with(dataset_id="dataset-123")

    def test_list_dataset_rows_missing_id(self):
        option_list = []
        with self.assertRaises(MissingRequirementException) as context:
            list_dataset_rows(option_list)
        self.assertEqual(str(context.exception), "Cannot list dataset rows without specifying id.")

    def test_get_dataset_row_success(self):
        option_list = [
            (Mock(spec=['name'], name="dataset_id"), "dataset-123"),
            (Mock(spec=['name'], name="row_id"), "row-456")
        ]
        for opt, _ in option_list:
            opt.name = opt._mock_name

        with patch('pygeai.evaluation.dataset.clients.EvaluationDatasetClient.get_dataset_row', return_value="Row detail") as mock_get:
            with patch('pygeai.core.utils.console.Console.write_stdout') as mock_stdout:
                get_dataset_row(option_list)
                mock_get.assert_called_once_with(dataset_id="dataset-123", dataset_row_id="row-456")

    def test_get_dataset_row_missing_dataset_id(self):
        option_list = [
            (Mock(spec=['name'], name="row_id"), "row-456")
        ]
        option_list[0][0].name = "row_id"

        with self.assertRaises(MissingRequirementException) as context:
            get_dataset_row(option_list)
        self.assertEqual(str(context.exception), "Cannot get dataset row without specifying id of dataset and row.")

    def test_get_dataset_row_missing_row_id(self):
        option_list = [
            (Mock(spec=['name'], name="dataset_id"), "dataset-123")
        ]
        option_list[0][0].name = "dataset_id"

        with self.assertRaises(MissingRequirementException) as context:
            get_dataset_row(option_list)
        self.assertEqual(str(context.exception), "Cannot get dataset row without specifying id of dataset and row.")

    def test_update_dataset_row_success(self):
        row_data = {
            "dataSetRowExpectedAnswer": "updated answer",
            "dataSetRowContextDocument": "updated context",
            "dataSetRowInput": "updated input"
        }
        option_list = [
            (Mock(spec=['name'], name="dataset_id"), "dataset-123"),
            (Mock(spec=['name'], name="row_id"), "row-456"),
            (Mock(spec=['name'], name="row"), json.dumps(row_data))
        ]
        for opt, _ in option_list:
            opt.name = opt._mock_name

        with patch('pygeai.evaluation.dataset.clients.EvaluationDatasetClient.update_dataset_row', return_value="Updated row") as mock_update:
            with patch('pygeai.core.utils.console.Console.write_stdout') as mock_stdout:
                update_dataset_row(option_list)
                mock_update.assert_called_once()

    def test_update_dataset_row_missing_dataset_id(self):
        option_list = [
            (Mock(spec=['name'], name="row_id"), "row-456")
        ]
        option_list[0][0].name = "row_id"

        with self.assertRaises(MissingRequirementException) as context:
            update_dataset_row(option_list)
        self.assertEqual(str(context.exception), "Cannot update dataset row without specifying id of dataset and row.")

    def test_delete_dataset_row_success(self):
        option_list = [
            (Mock(spec=['name'], name="dataset_id"), "dataset-123"),
            (Mock(spec=['name'], name="row_id"), "row-456")
        ]
        for opt, _ in option_list:
            opt.name = opt._mock_name

        with patch('pygeai.evaluation.dataset.clients.EvaluationDatasetClient.delete_dataset_row', return_value="Deleted row") as mock_delete:
            with patch('pygeai.core.utils.console.Console.write_stdout') as mock_stdout:
                delete_dataset_row(option_list)
                mock_delete.assert_called_once_with(dataset_id="dataset-123", dataset_row_id="row-456")

    def test_delete_dataset_row_missing_dataset_id(self):
        option_list = [
            (Mock(spec=['name'], name="row_id"), "row-456")
        ]
        option_list[0][0].name = "row_id"

        with self.assertRaises(MissingRequirementException) as context:
            delete_dataset_row(option_list)
        self.assertEqual(str(context.exception), "Cannot delete dataset row without specifying id of dataset and row.")

    # Dataset Row Expected Source Tests
    def test_create_dataset_row_expected_source(self):
        source_data = {
            "dataSetExpectedSourceName": "source1",
            "dataSetExpectedSourceValue": "value1"
        }
        option_list = [
            (Mock(spec=['name'], name="dataset_id"), "dataset-123"),
            (Mock(spec=['name'], name="row_id"), "row-456"),
            (Mock(spec=['name'], name="expected_source"), json.dumps(source_data))
        ]
        for opt, _ in option_list:
            opt.name = opt._mock_name

        with patch('pygeai.evaluation.dataset.clients.EvaluationDatasetClient.create_expected_source', return_value="Created source") as mock_create:
            with patch('pygeai.core.utils.console.Console.write_stdout') as mock_stdout:
                create_dataset_row_expected_source(option_list)
                mock_create.assert_called_once()

    def test_list_dataset_row_expected_sources(self):
        option_list = [
            (Mock(spec=['name'], name="dataset_id"), "dataset-123"),
            (Mock(spec=['name'], name="row_id"), "row-456")
        ]
        for opt, _ in option_list:
            opt.name = opt._mock_name

        with patch('pygeai.evaluation.dataset.clients.EvaluationDatasetClient.list_expected_sources', return_value="Sources list") as mock_list:
            with patch('pygeai.core.utils.console.Console.write_stdout') as mock_stdout:
                list_dataset_row_expected_sources(option_list)
                mock_list.assert_called_once()

    def test_update_dataset_rows_file(self):
        option_list = [
            (Mock(spec=['name'], name="dataset_id"), "dataset-123"),
            (Mock(spec=['name'], name="dataset_rows_file"), "/path/to/rows.json")
        ]
        for opt, _ in option_list:
            opt.name = opt._mock_name

        with patch('pygeai.evaluation.dataset.clients.EvaluationDatasetClient.upload_dataset_rows_file', return_value="Updated rows") as mock_update:
            with patch('pygeai.core.utils.console.Console.write_stdout') as mock_stdout:
                update_dataset_rows_file(option_list)
                mock_update.assert_called_once()

    # Evaluation Plan Commands Tests
    def test_list_evaluation_plans(self):
        with patch('pygeai.evaluation.plan.clients.EvaluationPlanClient.list_evaluation_plans', return_value="Plans list") as mock_list:
            with patch('pygeai.core.utils.console.Console.write_stdout') as mock_stdout:
                list_evaluation_plans()
                mock_list.assert_called_once()

    def test_create_evaluation_plan_success(self):
        option_list = [
            (Mock(spec=['name'], name="evaluation_plan_name"), "Test Plan"),
            (Mock(spec=['name'], name="evaluation_plan_type"), "Test Description"),
            (Mock(spec=['name'], name="evaluation_plan_assistant_id"), "1")
        ]
        for opt, _ in option_list:
            opt.name = opt._mock_name

        with patch('pygeai.evaluation.plan.clients.EvaluationPlanClient.create_evaluation_plan', return_value="Created plan") as mock_create:
            with patch('pygeai.core.utils.console.Console.write_stdout') as mock_stdout:
                create_evaluation_plan(option_list)
                mock_create.assert_called_once()

    def test_create_evaluation_plan_missing_name(self):
        option_list = []
        with self.assertRaises(MissingRequirementException) as context:
            create_evaluation_plan(option_list)
        self.assertEqual(str(context.exception), "Cannot create evaluation plan without specifying evaluation plan name and type")

    def test_get_evaluation_plan_success(self):
        option_list = [
            (Mock(spec=['name'], name="evaluation_plan_id"), "plan-123")
        ]
        option_list[0][0].name = "evaluation_plan_id"

        with patch('pygeai.evaluation.plan.clients.EvaluationPlanClient.get_evaluation_plan', return_value="Plan detail") as mock_get:
            with patch('pygeai.core.utils.console.Console.write_stdout') as mock_stdout:
                get_evaluation_plan(option_list)
                mock_get.assert_called_once_with(evaluation_plan_id="plan-123")

    def test_get_evaluation_plan_missing_id(self):
        option_list = []
        with self.assertRaises(MissingRequirementException) as context:
            get_evaluation_plan(option_list)
        self.assertEqual(str(context.exception), "Cannot get evaluation plan without specifying id")

    def test_update_evaluation_plan_success(self):
        option_list = [
            (Mock(spec=['name'], name="evaluation_plan_id"), "plan-123"),
            (Mock(spec=['name'], name="evaluation_plan_name"), "Updated Plan")
        ]
        for opt, _ in option_list:
            opt.name = opt._mock_name

        with patch('pygeai.evaluation.plan.clients.EvaluationPlanClient.update_evaluation_plan', return_value="Updated plan") as mock_update:
            with patch('pygeai.core.utils.console.Console.write_stdout') as mock_stdout:
                update_evaluation_plan(option_list)
                mock_update.assert_called_once()

    def test_update_evaluation_plan_missing_id(self):
        option_list = [
            (Mock(spec=['name'], name="evaluation_plan_name"), "Updated Plan")
        ]
        option_list[0][0].name = "plan_name"

        with self.assertRaises(MissingRequirementException) as context:
            update_evaluation_plan(option_list)
        self.assertEqual(str(context.exception), "Cannot update evaluation plan without specifying id")

    def test_delete_evaluation_plan_success(self):
        option_list = [
            (Mock(spec=['name'], name="evaluation_plan_id"), "plan-123")
        ]
        option_list[0][0].name = "evaluation_plan_id"

        with patch('pygeai.evaluation.plan.clients.EvaluationPlanClient.delete_evaluation_plan', return_value="Deleted") as mock_delete:
            with patch('pygeai.core.utils.console.Console.write_stdout') as mock_stdout:
                delete_evaluation_plan(option_list)
                mock_delete.assert_called_once_with(evaluation_plan_id="plan-123")

    def test_delete_evaluation_plan_missing_id(self):
        option_list = []
        with self.assertRaises(MissingRequirementException) as context:
            delete_evaluation_plan(option_list)
        self.assertEqual(str(context.exception), "Cannot delete evaluation plan without specifying id")

    # Evaluation Plan System Metric Tests
    def test_list_evaluation_plan_system_metrics_success(self):
        option_list = [
            (Mock(spec=['name'], name="evaluation_plan_id"), "plan-123")
        ]
        option_list[0][0].name = "evaluation_plan_id"

        with patch('pygeai.evaluation.plan.clients.EvaluationPlanClient.list_evaluation_plan_system_metrics', return_value="Metrics list") as mock_list:
            with patch('pygeai.core.utils.console.Console.write_stdout') as mock_stdout:
                list_evaluation_plan_system_metrics(option_list)
                mock_list.assert_called_once_with(evaluation_plan_id="plan-123")

    def test_list_evaluation_plan_system_metrics_missing_id(self):
        option_list = []
        with self.assertRaises(MissingRequirementException) as context:
            list_evaluation_plan_system_metrics(option_list)
        self.assertEqual(str(context.exception), "Cannot list evaluation plan's system metrics without specifying id")

    def test_add_evaluation_plan_system_metric_success(self):
        option_list = [
            (Mock(spec=['name'], name="evaluation_plan_id"), "plan-123"),
            (Mock(spec=['name'], name="system_metric_id"), "metric-456"),
            (Mock(spec=['name'], name="system_metric_weight"), "1.0")
        ]
        for opt, _ in option_list:
            opt.name = opt._mock_name

        with patch('pygeai.evaluation.plan.clients.EvaluationPlanClient.add_evaluation_plan_system_metric', return_value="Added metric") as mock_add:
            with patch('pygeai.core.utils.console.Console.write_stdout') as mock_stdout:
                add_evaluation_plan_system_metric(option_list)
                mock_add.assert_called_once()

    def test_add_evaluation_plan_system_metric_missing_plan_id(self):
        option_list = [
            (Mock(spec=['name'], name="system_metric_id"), "metric-456")
        ]
        option_list[0][0].name = "system_metric_id"

        with self.assertRaises(MissingRequirementException) as context:
            add_evaluation_plan_system_metric(option_list)
        self.assertEqual(str(context.exception), "Cannot add evaluation plan's system metrics without specifying id")

    def test_get_evaluation_plan_system_metric_success(self):
        option_list = [
            (Mock(spec=['name'], name="evaluation_plan_id"), "plan-123"),
            (Mock(spec=['name'], name="system_metric_id"), "metric-456")
        ]
        for opt, _ in option_list:
            opt.name = opt._mock_name

        with patch('pygeai.evaluation.plan.clients.EvaluationPlanClient.get_evaluation_plan_system_metric', return_value="Metric detail") as mock_get:
            with patch('pygeai.core.utils.console.Console.write_stdout') as mock_stdout:
                get_evaluation_plan_system_metric(option_list)
                mock_get.assert_called_once_with(evaluation_plan_id="plan-123", system_metric_id="metric-456")

    def test_get_evaluation_plan_system_metric_missing_plan_id(self):
        option_list = [
            (Mock(spec=['name'], name="system_metric_id"), "metric-456")
        ]
        option_list[0][0].name = "system_metric_id"

        with self.assertRaises(MissingRequirementException) as context:
            get_evaluation_plan_system_metric(option_list)
        self.assertEqual(str(context.exception), "Cannot retrieve evaluation plan's system metric without specifying both ids")

    def test_update_evaluation_plan_system_metric_success(self):
        option_list = [
            (Mock(spec=['name'], name="evaluation_plan_id"), "plan-123"),
            (Mock(spec=['name'], name="system_metric_id"), "metric-456"),
            (Mock(spec=['name'], name="system_metric_weight"), "2.0")
        ]
        for opt, _ in option_list:
            opt.name = opt._mock_name

        with patch('pygeai.evaluation.plan.clients.EvaluationPlanClient.update_evaluation_plan_system_metric', return_value="Updated metric") as mock_update:
            with patch('pygeai.core.utils.console.Console.write_stdout') as mock_stdout:
                update_evaluation_plan_system_metric(option_list)
                mock_update.assert_called_once()

    def test_delete_evaluation_plan_system_metric_success(self):
        option_list = [
            (Mock(spec=['name'], name="evaluation_plan_id"), "plan-123"),
            (Mock(spec=['name'], name="system_metric_id"), "metric-456")
        ]
        for opt, _ in option_list:
            opt.name = opt._mock_name

        with patch('pygeai.evaluation.plan.clients.EvaluationPlanClient.delete_evaluation_plan_system_metric', return_value="Deleted metric") as mock_delete:
            with patch('pygeai.core.utils.console.Console.write_stdout') as mock_stdout:
                delete_evaluation_plan_system_metric(option_list)
                mock_delete.assert_called_once_with(evaluation_plan_id="plan-123", system_metric_id="metric-456")

    # System Metrics Tests
    def test_list_system_metrics(self):
        with patch('pygeai.evaluation.plan.clients.EvaluationPlanClient.list_system_metrics', return_value="System metrics list") as mock_list:
            with patch('pygeai.core.utils.console.Console.write_stdout') as mock_stdout:
                list_system_metrics()
                mock_list.assert_called_once()

    def test_get_system_metric_success(self):
        option_list = [
            (Mock(spec=['name'], name="system_metric_id"), "metric-456")
        ]
        option_list[0][0].name = "system_metric_id"

        with patch('pygeai.evaluation.plan.clients.EvaluationPlanClient.get_system_metric', return_value="System metric detail") as mock_get:
            with patch('pygeai.core.utils.console.Console.write_stdout') as mock_stdout:
                get_system_metric(option_list)
                mock_get.assert_called_once_with(system_metric_id="metric-456")

    def test_get_system_metric_missing_id(self):
        option_list = []
        with self.assertRaises(MissingRequirementException) as context:
            get_system_metric(option_list)
        self.assertEqual(str(context.exception), "Cannot retrieve system metric without specifying id")

    # Execution Tests
    def test_execute_evaluation_plan_success(self):
        option_list = [
            (Mock(spec=['name'], name="evaluation_plan_id"), "plan-123"),
            (Mock(spec=['name'], name="dataset_id"), "dataset-123"),
            (Mock(spec=['name'], name="assistant_id"), "assistant-123")
        ]
        for opt, _ in option_list:
            opt.name = opt._mock_name

        with patch('pygeai.evaluation.plan.clients.EvaluationPlanClient.execute_evaluation_plan', return_value="Execution result") as mock_execute:
            with patch('pygeai.core.utils.console.Console.write_stdout') as mock_stdout:
                execute_evaluation_plan(option_list)
                mock_execute.assert_called_once()

    def test_execute_evaluation_plan_missing_plan_id(self):
        option_list = []
        with self.assertRaises(MissingRequirementException) as context:
            execute_evaluation_plan(option_list)
        self.assertEqual(str(context.exception), "Cannot execute evaluation plan without specifying id")

    # Result Tests
    def test_list_evaluation_results(self):
        option_list = [
            (Mock(spec=['name'], name="evaluation_plan_id"), "plan-123")
        ]
        option_list[0][0].name = "evaluation_plan_id"

        with patch('pygeai.evaluation.result.clients.EvaluationResultClient.list_evaluation_results', return_value="Results list") as mock_list:
            with patch('pygeai.core.utils.console.Console.write_stdout') as mock_stdout:
                list_evaluation_results(option_list)
                mock_list.assert_called_once()

    def test_get_evaluation_result_success(self):
        option_list = [
            (Mock(spec=['name'], name="evaluation_result_id"), "result-123")
        ]
        option_list[0][0].name = "evaluation_result_id"

        with patch('pygeai.evaluation.result.clients.EvaluationResultClient.get_evaluation_result', return_value="Result detail") as mock_get:
            with patch('pygeai.core.utils.console.Console.write_stdout') as mock_stdout:
                get_evaluation_result(option_list)
                mock_get.assert_called_once_with(evaluation_result_id="result-123")

    def test_get_evaluation_result_missing_id(self):
        option_list = []
        with self.assertRaises(MissingRequirementException) as context:
            get_evaluation_result(option_list)
        self.assertEqual(str(context.exception), "Cannot get evaluation results without specifying id")


if __name__ == '__main__':
    unittest.main()



from typing import List, Union, Optional

from pydantic.main import BaseModel


class TokenDetails(BaseModel):
    reasoning_tokens: Optional[int] = None
    cached_tokens: Optional[int] = None


class EmbeddingData(BaseModel):
    index: int
    embedding: Union[List[float], str]
    object: str


class UsageInfo(BaseModel):
    prompt_tokens: int
    total_cost: float
    total_tokens: int
    currency: str
    prompt_cost: float
    completion_tokens_details: Optional[TokenDetails] = None
    prompt_tokens_details: Optional[TokenDetails] = None


class EmbeddingResponse(BaseModel):
    data: List[EmbeddingData]
    usage: UsageInfo
    model: str
    object: str



from pygeai.evaluation.dataset.clients import EvaluationDatasetClient

client = EvaluationDatasetClient()

response = client.create_dataset(
    dataset_name="My Test Dataset",
    dataset_description="A dataset for testing purposes",
    dataset_type="T",
    dataset_active=True,
    rows=[
        {
            "dataSetRowInput": "What is the capital of France?",
            "dataSetRowExpectedAnswer": "Paris",
            "dataSetRowContextDocument": "France is a country in Europe."
        },
        {
            "dataSetRowInput": "What is 2+2?",
            "dataSetRowExpectedAnswer": "4",
            "dataSetRowContextDocument": ""
        }
    ]
)

print(f"Created dataset: {response}")
dataset_id = response.get('dataSetId')
print(f"Dataset ID: {dataset_id}")



from unittest import TestCase
import unittest
from pygeai.lab.managers import AILabManager
from pygeai.lab.models import Tool, FilterSettings
from pygeai.core.common.exceptions import APIResponseError
import copy

ai_lab_manager: AILabManager

class TestAILabGetToolIntegration(TestCase):   

    def setUp(self):
        self.ai_lab_manager = AILabManager()
        self.tool_id = "e3e4d64f-ce52-467e-90a9-aa4d08425e82"
        self.filter_settings = FilterSettings(
            revision="0",
            version="0"
        )

    def __get_tool(self, tool_id=None, filter_settings: FilterSettings = None):
        return self.ai_lab_manager.get_tool(
            tool_id=self.tool_id if tool_id is None else tool_id,
            filter_settings=self.filter_settings if filter_settings is None else filter_settings
        )

    def test_get_tool(self):
        tool = self.__get_tool()
        self.assertIsInstance(tool, Tool, "Expected a tool") 
 

    @unittest.skip("Skipped: Validate that when no tool_id is provided, the complete tool list is returned")
    def test_get_tool_no_tool_id(self):
        with self.assertRaises(Exception) as context:
            self.__get_tool(tool_id="")
         
    
    def test_get_tool_invalid_tool_id(self):
        invalid_id = "0026e53d-ea78-4cac-af9f-12650invalid"
        with self.assertRaises(APIResponseError) as context:
            self.__get_tool(tool_id=invalid_id)
        self.assertIn(
            f"Tool not found [IdOrName= {invalid_id}].",
            str(context.exception),
            "Expected an error for invalid tool id"
        )


    def test_get_tool_no_revision(self):
        filter_settings = copy.deepcopy(self.filter_settings)
        filter_settings.revision = None
        tool = self.__get_tool(filter_settings=filter_settings)

        self.assertIsInstance(tool, Tool, "Expected a tool")
        self.assertGreaterEqual(tool.revision, 1, "Expected tool revision to be the latest")

    
    def test_get_tool_by_revision(self):
        filter_settings = copy.deepcopy(self.filter_settings)
        filter_settings.revision = "6"
        tool = self.__get_tool(filter_settings=filter_settings)

        self.assertEqual(6, tool.revision, "Expected agent revision to be 6")


    def test_get_tool_by_earlier_revision(self):
        filter_settings = copy.deepcopy(self.filter_settings)
        filter_settings.revision = "2"
        with self.assertRaises(APIResponseError) as context:
            self.__get_tool(filter_settings=filter_settings)
        self.assertIn(
            f"Requested revision not found [revision={filter_settings.revision}].",
            str(context.exception),
            "Expected an error for revision not found"
        )

    #TODO: The API is returning the version of the tool, but the sdk does not
    def test_get_tool_no_version(self):
        filter_settings = copy.deepcopy(self.filter_settings)
        filter_settings.version = None
        tool = self.__get_tool(filter_settings=filter_settings)

        self.assertIsInstance(tool, Tool, "Expected a tool") 


    #TODO: The API is returning the version of the tool, but the sdk does not      
    def test_get_tool_by_version(self):
        filter_settings = copy.deepcopy(self.filter_settings)
        filter_settings.version = "1"
        tool = self.__get_tool(filter_settings=filter_settings)
        
        self.assertIsInstance(tool, Tool, "Expected a tool") 




class GEAIException(Exception):
    """Base class for all PyGEAI exceptions."""
    pass


class UnknownArgumentError(GEAIException):
    """Raised when an unknown or invalid command/option is provided"""
    
    def __init__(self, message: str, arg: str = None, available_commands=None, available_options=None):
        """
        Initialize an UnknownArgumentError with context.
        
        :param message: str - The error message.
        :param arg: str - The unknown argument that was provided.
        :param available_commands: list - Available commands for suggestion.
        :param available_options: list - Available options for suggestion.
        """
        super().__init__(message)
        self.arg = arg
        self.available_commands = available_commands
        self.available_options = available_options


class MissingRequirementException(GEAIException):
    """Raised when a required parameter or argument is missing"""
    pass


class WrongArgumentError(GEAIException):
    """Raised when arguments are incorrectly formatted or invalid"""
    pass


class ValidationError(WrongArgumentError):
    """
    Raised when input validation fails with detailed context.
    
    Extends WrongArgumentError to provide structured validation errors
    with field-specific information and examples.
    """
    def __init__(
        self, 
        message: str, 
        field: str = None, 
        expected: str = None,
        received: str = None,
        example: str = None
    ):
        """
        Initialize a ValidationError with detailed context.

        :param message: str - The main error message.
        :param field: str - Name of the field that failed validation.
        :param expected: str - Description of what was expected.
        :param received: str - Description of what was received.
        :param example: str - Example of valid input.
        """
        super().__init__(message)
        self.field = field
        self.expected = expected
        self.received = received
        self.example = example
    
    def __str__(self) -> str:
        """
        Format the error message with all available context.

        :return: str - Formatted error message with field details.
        """
        parts = [super().__str__()]
        
        if self.field:
            parts.append(f"  Field: {self.field}")
        if self.expected:
            parts.append(f"  Expected: {self.expected}")
        if self.received:
            parts.append(f"  Received: {self.received}")
        if self.example:
            parts.append(f"  Example: {self.example}")
            
        return "\n".join(parts)


class ServerResponseError(GEAIException):
    """Raised when the server returns an error response"""
    pass


class APIError(GEAIException):
    """Raised when an API request fails"""
    pass


class InvalidPathException(GEAIException):
    """Raised when a file or directory path is invalid or not found"""
    pass


class InvalidJSONException(GEAIException):
    """Raised when JSON data cannot be parsed or is malformed"""
    pass


class InvalidAPIResponseException(GEAIException):
    """Raised when the API response format is unexpected or invalid"""
    pass


class InvalidResponseException(GEAIException):
    """Raised when a response cannot be retrieved or processed"""
    pass


class InvalidAgentException(GEAIException):
    """Raised when an agent cannot be retrieved, validated, or is misconfigured"""
    pass


class APIResponseError(GEAIException):
    """Raised when there is an error in the API response"""
    pass




from pygeai.evaluation.dataset.clients import EvaluationDatasetClient

client = EvaluationDatasetClient()

dataset_id = "your-dataset-id-here"
row_id = "your-row-id-here"

response = client.create_filter_variable(
    dataset_id=dataset_id,
    dataset_row_id=row_id,
    metadata_type="V",
    filter_variable_key="category",
    filter_variable_value="science",
    filter_variable_operator="="
)

print(f"Created filter variable: {response}")
filter_var_id = response.get('dataSetRowFilterVarId')
print(f"Filter Variable ID: {filter_var_id}")



from pygeai.lab.managers import AILabManager
from pygeai.lab.models import ReasoningStrategy

manager = AILabManager()

result = manager.get_reasoning_strategy(
    reasoning_strategy_id="2b757122-3e36-499d-909e-87074c3afc94"
)

print(f"Retrieved: {result.name}, ID: {result.id}")



from unittest import TestCase
from pygeai.chat.clients import ChatClient

chat_client: ChatClient

class TestChatGenerateImageIntegration(TestCase):    

    def setUp(self):
        self.chat_client = ChatClient()
        self.new_image = self.__load_image()


    def __load_image(self):
        return {
            "model": "openai/gpt-image-1",
            "prompt": "generate an image of a futuristic city skyline at sunset",
            "n": 1,
            "quality": "high",
            "size": "1024x1536"
        }
    

    def __generate_image(self, image = None):
        image = image if image is not None else self.new_image
        return self.chat_client.generate_image(
            model=image["model"],
            prompt=image["prompt"],
            n=image["n"],
            quality=image["quality"],
            size=image["size"],
            aspect_ratio= image["aspect_ratio"] if "aspect_ratio" in image else None
        )

    
    def test_generate_image(self):
        created_image = self.__generate_image()
        self.assertEqual(len(created_image["data"]), 1, "Expected an image to be generated")


    def test_generate_image_invalid_model(self):
        self.new_image["model"] = "openai/gpt-image-10",
        created_image = self.__generate_image()

        self.assertEqual(
            created_image["error"]["code"], 400,
            "Expected a 400 code for invalid model"
        )
        self.assertEqual(
            created_image["error"]["message"],
            'Provider \'["openai\' does not exists.', 
            "Expected an error message when model does not exists"
        )


    def test_generate_image_no_model(self):
        self.new_image["model"] = ""
        created_image = self.__generate_image()

        self.assertEqual(
            created_image["error"]["message"],
            "LLM Provider NOT provided. Pass in the LLM provider you are trying to call", 
            "Expected an error message when no model is provided"
        )


    def test_generate_image_no_prompt(self):
        self.new_image["prompt"] = ""
        created_image = self.__generate_image()

        self.assertEqual(
            created_image["error"]["type"],
            "invalid_request_error",
            "Expected a 400 code for no model"
        )
        self.assertEqual(
            created_image["error"]["param"], "prompt", 
            "Expected an error message when no model is provided"
        )


    def test_generate_image_specific_n(self):
        self.new_image["n"] = 2

        created_image = self.__generate_image()
        self.assertEqual(len(created_image["data"]), 2, "Expected two images to be generated")


    def test_generate_image_no_n(self):
        self.new_image["n"] = None # default is 1

        created_image = self.__generate_image()
        self.assertEqual(len(created_image["data"]), 1, "Expected an image to be generated")


    def test_generate_image_no_supported_n(self):
        self.new_image["model"] = "openai/dall-e-3"
        self.new_image["n"] = 5

        created_image = self.__generate_image()
        self.assertIn(
            "Invalid 'n': integer above maximum value",
            created_image["error"]["message"],
            "Expected an error message when n is not supported by the model"
        )


    def test_generate_image_no_quality(self):
        self.new_image["quality"] = ""

        created_image = self.__generate_image()
        self.assertIn(
            "Invalid value: ''. Supported values are: 'low', 'medium', 'high', and 'auto'",
            created_image["error"]["message"],
            "Expected an error message when quality is not provided"
        )
    

    def test_generate_image_no_supported_quality(self):
        self.new_image["model"] = "openai/dall-e-3"

        created_image = self.__generate_image()
        self.assertIn(
            "Invalid value: 'high'. Supported values are: 'standard' and 'hd'",
            created_image["error"]["message"],
            "Expected an error message when quality is not supported by the model"
        )


    def test_generate_image_no_size(self):
        self.new_image["size"] = ""

        created_image = self.__generate_image()
        self.assertIn(
            "Invalid value: ''. Supported values are: '1024x1024', '1024x1536', '1536x1024', and 'auto'",
            created_image["error"]["message"],
            "Expected an error message when no size is provided"
        )


    def test_generate_image_no_supported_size(self):
        self.new_image["size"] = 1024
        self.new_image["quality"] = None

        created_image = self.__generate_image()
        self.assertIn(
            "Invalid type for 'size': expected one of '1024x1024', '1024x1536', '1536x1024', or 'auto', but got an integer instead",
            created_image["error"]["message"],
            "Expected an error message when no size is provided"
        )
    

    def test_generate_image_with_aspect_ratio(self):
        self.new_image["model"] = "vertex_ai/imagen-3.0-generate-001"
        self.new_image["aspect_ratio"] = "4:3"
        self.new_image["quality"] = None

        created_image = self.__generate_image()
        self.assertEqual(len(created_image["data"]), 1, "Expected an image to be generated")


from pygeai.analytics.managers import AnalyticsManager

manager = AnalyticsManager()

response = manager.get_average_cost_per_request(
    start_date="2025-01-01",
    end_date="2026-01-31"
)

print(f"Average cost per request: ${response.averageCost:.4f}")



import csv
from pygeai.organization.clients import OrganizationClient

client = OrganizationClient()

batch_file = "project_members.csv"

successful = 0
failed = 0
errors = []

with open(batch_file, 'r') as f:
    csv_reader = csv.reader(f)
    for line_num, row in enumerate(csv_reader, start=1):
        if len(row) < 3:
            error_msg = f"Line {line_num}: Invalid format - expected at least 3 columns (project_id, email, role1, ...)"
            errors.append(error_msg)
            failed += 1
            continue

        project_id = row[0].strip()
        email = row[1].strip()
        roles = [r.strip() for r in row[2:] if r.strip()]

        if not (project_id and email and roles):
            error_msg = f"Line {line_num}: Missing required fields"
            errors.append(error_msg)
            failed += 1
            continue

        try:
            result = client.add_project_member(project_id, email, roles)
            print(f"Successfully added {email} to project {project_id}")
            successful += 1
        except Exception as e:
            error_msg = f"Line {line_num}: Failed to add {email} to project {project_id}: {str(e)}"
            errors.append(error_msg)
            failed += 1

print(f"\nBatch processing complete: {successful} successful, {failed} failed")
if errors:
    print("\nErrors:")
    for error in errors:
        print(f"  - {error}")



import json
import platform
import os
import subprocess
import sys
from importlib import resources
from pathlib import Path

from pygeai import logger
from pygeai.chat.clients import ChatClient
from pygeai.chat.iris import Iris
from pygeai.chat.session import AgentChatSession
from pygeai.cli.commands import Command, Option, ArgumentsEnum
from pygeai.cli.commands.builders import build_help_text
from pygeai.cli.commands.common import get_messages, get_boolean_value, get_penalty_float_value
from pygeai.cli.commands.lab.utils import get_project_id
from pygeai.cli.texts.help import CHAT_HELP_TEXT
from pygeai.core.common.exceptions import MissingRequirementException, WrongArgumentError, InvalidAgentException
from prompt_toolkit import PromptSession
from prompt_toolkit.history import InMemoryHistory

from pygeai.core.utils.console import Console
from pygeai.lab.agents.clients import AgentClient


def show_help():
    """
    Displays help text in stdout
    """
    help_text = build_help_text(chat_commands, CHAT_HELP_TEXT)
    Console.write_stdout(help_text)


def get_chat_completion(option_list: list):
    model = None
    message_list = []
    stream = False
    temperature = None
    max_tokens = None
    thread_id = None
    frequency_penalty = None
    presence_penalty = None
    top_p = None
    stop = None
    response_format = None
    tools = None
    tool_choice = None
    logprobs = None
    top_logprobs = None
    seed = None
    stream_options = None
    store = None
    metadata = None
    user = None
    reasoning_effort = None

    for option_flag, option_arg in option_list:
        if option_flag.name == "model":
            model = option_arg
        if option_flag.name == "messages":
            try:
                message_json = json.loads(option_arg)
                if isinstance(message_json, list):
                    message_list = message_json
                elif isinstance(message_json, dict):
                    message_list.append(message_json)
            except Exception as e:
                raise WrongArgumentError(
                    "Each message must be in json format: '{\"role\": \"user\", \"content\": \"message content\"}' "
                    "It can be a dictionary or a list of dictionaries. Each dictionary must contain role and content")
        if option_flag.name == "stream":
            if option_arg:
                stream = get_boolean_value(option_arg)
        if option_flag.name == "temperature":
            temperature = float(option_arg) if option_arg is not None else None
        if option_flag.name == "max_tokens":
            max_tokens = int(option_arg) if option_arg is not None else None
        if option_flag.name == "thread_id":
            thread_id = option_arg
        if option_flag.name == "frequency_penalty":
            if option_arg:
                frequency_penalty = get_penalty_float_value(option_arg)
        if option_flag.name == "presence_penalty":
            if option_arg:
                presence_penalty = get_penalty_float_value(option_arg)
        if option_flag.name == "top_p":
            top_p = float(option_arg) if option_arg else None
        if option_flag.name == "stop":
            try:
                stop = json.loads(option_arg) if option_arg else None
            except json.JSONDecodeError:
                stop = option_arg
        if option_flag.name == "response_format":
            try:
                response_format = json.loads(option_arg) if option_arg else None
            except json.JSONDecodeError:
                raise WrongArgumentError("response_format must be a valid JSON object")
        if option_flag.name == "tools":
            try:
                tools = json.loads(option_arg) if option_arg else None
            except json.JSONDecodeError:
                raise WrongArgumentError("tools must be a valid JSON array")
        if option_flag.name == "tool_choice":
            try:
                tool_choice = json.loads(option_arg) if option_arg else None
            except json.JSONDecodeError:
                tool_choice = option_arg
        if option_flag.name == "logprobs":
            logprobs = get_boolean_value(option_arg) if option_arg else None
        if option_flag.name == "top_logprobs":
            top_logprobs = int(option_arg) if option_arg else None
        if option_flag.name == "seed":
            seed = int(option_arg) if option_arg else None
        if option_flag.name == "stream_options":
            try:
                stream_options = json.loads(option_arg) if option_arg else None
            except json.JSONDecodeError:
                raise WrongArgumentError("stream_options must be a valid JSON object")
        if option_flag.name == "store":
            store = get_boolean_value(option_arg) if option_arg else None
        if option_flag.name == "metadata":
            try:
                metadata = json.loads(option_arg) if option_arg else None
            except json.JSONDecodeError:
                raise WrongArgumentError("metadata must be a valid JSON object")
        if option_flag.name == "user":
            user = option_arg
        if option_flag.name == "reasoning_effort":
            reasoning_effort = option_arg

    messages = get_messages(message_list)

    if not (model and messages):
        raise MissingRequirementException("Cannot perform chat completion without specifying model and messages")

    client = ChatClient()
    result = client.chat_completion(
        model=model,
        messages=messages,
        stream=stream,
        temperature=temperature,
        max_tokens=max_tokens,
        thread_id=thread_id,
        frequency_penalty=frequency_penalty,
        presence_penalty=presence_penalty,
        top_p=top_p,
        stop=stop,
        response_format=response_format,
        tools=tools,
        tool_choice=tool_choice,
        logprobs=logprobs,
        top_logprobs=top_logprobs,
        seed=seed,
        stream_options=stream_options,
        store=store,
        metadata=metadata,
        user=user,
        reasoning_effort=reasoning_effort
    )
    if stream:
        Console.write_stdout("Streaming chat completion:")
        for chunk in result:
            Console.write_stdout(f"{chunk}", end="")
            sys.stdout.flush()
        Console.write_stdout()
    else:
        Console.write_stdout(f"Chat completion detail: \n{result}\n")


chat_completion_options = [
    Option(
        "model",
        ["--model", "-m"],
        "The model needs to address the assistant type and name or bot_id, depending on the Type. Then, the parameters"
        " will vary depending on the type. Its format is as follows: \n"
        "\t\"model\": \"saia:<assistant_type>:<assistant_name>|<bot_id>\"",
        True
    ),
    Option(
        "messages",
        ["--messages", "--msg"],
        "The messages element defines the desired messages to be added. The minimal value needs to be the following, "
        "where the content details the user input.\n"
        "\t{ \n"
        "\t\t\"role\": \"string\", /* user, system and may support others depending on the selected model */ \n"
        "\t\t\"content\": \"string\" \n"
        "\t}\n",
        True
    ),
    Option(
        "stream",
        ["--stream"],
        "If response should be streamed. Possible values: 0: OFF; 1: ON",
        True
    ),
    Option(
        "temperature",
        ["--temperature", "--temp"],
        "Float value to set volatility of the assistant's answers (between 0 and 2)",
        True
    ),
    Option(
        "max_tokens",
        ["--max-tokens"],
        "Integer value to set max tokens to use",
        True
    ),
    Option(
        "thread_id",
        ["--thread-id"],
        "Optional UUID for conversation identifier",
        True
    ),
    Option(
        "frequency_penalty",
        ["--frequency-penalty"],
        "Optional number between -2.0 and 2.0. Positive values penalize new tokens based on their existing frequency "
        "in the text so far, decreasing the model's likelihood to repeat the same line verbatim.",
        True
    ),
    Option(
        "presence_penalty",
        ["--presence-penalty"],
        "Optional number between -2.0 and 2.0. Positive values penalize new tokens based on whether they appear in "
        "the text so far, increasing the model's likelihood to talk about new topics.",
        True
    ),
    Option(
        "top_p",
        ["--top-p"],
        "Optional float value for nucleus sampling, where the model considers tokens with top_p probability mass "
        "(between 0 and 1). An alternative to temperature.",
        True
    ),
    Option(
        "stop",
        ["--stop"],
        "Optional string or JSON array of up to 4 sequences where the API will stop generating further tokens.",
        True
    ),
    Option(
        "response_format",
        ["--response-format"],
        "Optional JSON object specifying the output format, e.g., {\"type\": \"json_schema\", \"json_schema\": {...}} "
        "for structured outputs.",
        True
    ),
    Option(
        "tools",
        ["--tools"],
        "Optional JSON array of tools (e.g., functions) the model may call.",
        True
    ),
    Option(
        "tool_choice",
        ["--tool-choice"],
        "Optional string (e.g., \"none\", \"auto\") or JSON object to control which tool is called.",
        True
    ),
    Option(
        "logprobs",
        ["--logprobs"],
        "Optional boolean to return log probabilities of output tokens. Possible values: 0: OFF; 1: ON",
        True
    ),
    Option(
        "top_logprobs",
        ["--top-logprobs"],
        "Optional integer (0-20) specifying the number of most likely tokens to return with log probabilities.",
        True
    ),
    Option(
        "seed",
        ["--seed"],
        "Optional integer for deterministic sampling (in Beta).",
        True
    ),
    Option(
        "stream_options",
        ["--stream-options"],
        "Optional JSON object for streaming options, e.g., {\"include_usage\": true}.",
        True
    ),
    Option(
        "store",
        ["--store"],
        "Optional boolean to store the output for model distillation or evals. Possible values: 0: OFF; 1: ON",
        True
    ),
    Option(
        "metadata",
        ["--metadata"],
        "Optional JSON object with up to 16 key-value pairs to attach to the object.",
        True
    ),
    Option(
        "user",
        ["--user"],
        "Optional string identifier for the end-user to monitor abuse.",
        True
    ),
    Option(
        "reasoning_effort",
        ["--reasoning-effort"],
        "Optional string to control the depth of reasoning applied by supported models. "
        "Possible values: 'low', 'medium', 'high'. Supported by OpenAI models from version 5, "
        "Claude models from version 4.1, and Gemini models from version 2.0.",
        True
    ),
]


def chat_with_iris():
    iris = Iris()
    messages = list()

    history = InMemoryHistory()
    session = PromptSession(">> Ask Iris: ", history=history)

    Console.write_stdout(f"#=================================================#")
    Console.write_stdout(f"#--------------------- IRIS ----------------------#")
    Console.write_stdout(f"#=================================================#")
    Console.write_stdout(f"# This is the start of your conversation with Iris. Type 'exit' or press Ctrl+C to close the chat.\n")
    Console.write_stdout("""- Iris: Hello! I'm Iris. I'll guide you step by step to create your agent.
First, we need to define some key details for your agent. You can specify its role and purpose or give it a name, and I'll help you set up the rest. Once we have that, we'll refine its knowledge and behavior.""")
    try:
        while (user_input := session.prompt()) != "exit":
            Console.write_stdout(f"- User: {user_input}")
            new_message = {
                "role": "user",
                "content": user_input
            }
            messages.append(new_message)

            result = iris.stream_answer(messages)
            answer = ""
            Console.write_stdout("- Iris: ")
            for chunk in result:
                answer += chunk
                Console.write_stdout(f"{chunk}", end="")
                sys.stdout.flush()
            Console.write_stdout()

            new_answer = {
                "role": "assistant",
                "content": answer
            }
            messages.append(new_answer)
    except KeyboardInterrupt:
        print("\nExiting chat...")
        sys.exit(0)

    except Exception as e:
        logger.error(f"Error chatting with Iris: {e}")
        Console.write_stderr("An unexpected error has occurred. Please contact the developers.")


def chat_with_agent(option_list: list):
    agent_name = None
    use_gui = False
    save_session_file = None
    restore_session_file = None
    for option_flag, option_arg in option_list:
        if option_flag.name == "agent_name":
            agent_name = option_arg
        if option_flag.name == "gui":
            use_gui = True
        if option_flag.name == "save_session":
            save_session_file = option_arg
        if option_flag.name == "restore_session":
            restore_session_file = option_arg

    if not agent_name:
        raise MissingRequirementException(f"Agent name must be specified.")

    project_id = get_project_id()
    agent_data = AgentClient(project_id=project_id).get_agent(agent_id=agent_name)
    if 'errors' in agent_data:
        raise InvalidAgentException(f"There is no agent with that name: {agent_data.get('errors')}")

    if use_gui:
        try:
            import streamlit
        except ImportError:
            logger.error("Streamlit not installed")
            Console.write_stderr("Streamlit is required for GUI mode. Install it with 'pip install streamlit'.")
            sys.exit(1)

        try:
            ui_path = resources.files("pygeai.chat").joinpath("ui.py")
            ui_file_path = str(ui_path)

            # Add the top-level project root to PYTHONPATH
            package_root = str(Path(ui_file_path).resolve().parents[2])
            env = os.environ.copy()
            env["PYTHONPATH"] = package_root + os.pathsep + env.get("PYTHONPATH", "")

            streamlit_cmd = [
                sys.executable, "-m", "streamlit", "run", ui_file_path,
                "--server.address", "127.0.0.1",
                "--", "--agent-name", agent_name
            ]

            if platform.system() == "Linux":
                streamlit_cmd.insert(5, "--server.headless=true")

            process = subprocess.Popen(
                streamlit_cmd,
                stdout=subprocess.PIPE,
                stderr=subprocess.PIPE,
                text=True
            )

            url = "http://localhost:8501"
            if platform.system() == "Linux":
                Console.write_stdout(f"Open Streamlit app at {url} (or next port like 8502 if 8501 is taken)")

            try:
                stdout, stderr = process.communicate()
                if stderr:
                    logger.error(f"Streamlit stderr:\n{stderr}")
                    Console.write_stderr(f"Streamlit error:\n{stderr}")
            except KeyboardInterrupt:
                process.terminate()
                Console.write_stdout("Streamlit stopped.")
                sys.exit(0)

        except FileNotFoundError:
            logger.error("Could not locate pygeai/chat/ui.py")
            Console.write_stderr("Streamlit UI file not found. Ensure pygeai is installed correctly.")
            sys.exit(1)
        except Exception as e:
            logger.error(f"Streamlit error: {e}")
            Console.write_stderr(
                f"Failed to launch Streamlit. Check port with 'lsof -i :8501' and kill any process, or try {url}.")
            sys.exit(1)
    else:
        chat_session = AgentChatSession(agent_name)
        messages = list()

        # Restore session if specified
        if restore_session_file:
            try:
                with open(restore_session_file, 'r') as f:
                    restored_data = json.load(f)
                    if isinstance(restored_data, list):
                        messages = restored_data
                        Console.write_stdout(f"Restored conversation from {restore_session_file}")
                        # Display restored conversation history
                        Console.write_stdout(f"#=================================================#")
                        Console.write_stdout(f"#    CHAT SESSION WITH {agent_name}               ")
                        Console.write_stdout(f"#=================================================#")
                        for msg in messages:
                            role = msg.get("role", "unknown")
                            content = msg.get("content", "")
                            if role == "assistant":
                                Console.write_stdout(f"- Agent: {content}")
                            elif role == "user":
                                Console.write_stdout(f"- User: {content}")
                    else:
                        raise WrongArgumentError("Session file must contain a list of messages in JSON format.")
            except FileNotFoundError:
                logger.error(f"Session file {restore_session_file} not found.")
                Console.write_stderr(f"Session file {restore_session_file} not found. Starting a new conversation.")
            except json.JSONDecodeError:
                logger.error(f"Invalid JSON format in {restore_session_file}.")
                Console.write_stderr(f"Invalid JSON format in {restore_session_file}. Starting a new conversation.")
            except Exception as e:
                logger.error(f"Error restoring session: {e}")
                Console.write_stderr(f"Error restoring session from {restore_session_file}. Starting a new conversation.")

        # If no session was restored or messages are empty, get the agent's introduction
        if not messages:
            introduction_message = chat_session.get_answer(
                ["You're about to speak to a user. Introduce yourself in a clear and concise manner, "
                 "stating who you are and what you do. Nothing else."]
            )

            if "Agent not found" in str(introduction_message):
                raise WrongArgumentError(
                    "The specified agent doesn't seem to exist. Please review the name and try again.")

            Console.write_stdout(f"#=================================================#")
            Console.write_stdout(f"#    CHAT SESSION WITH {agent_name}               ")
            Console.write_stdout(f"#=================================================#")
            Console.write_stdout(introduction_message)
            messages.append({"role": "assistant", "content": introduction_message})

        history = InMemoryHistory()
        session = PromptSession(f">> Ask {agent_name}: ", history=history)

        # Save session function
        def save_session(messages, file_path):
            try:
                with open(file_path, 'w') as f:
                    json.dump(messages, f, indent=2)
                logger.info(f"Session saved to {file_path}")
            except Exception as e:
                logger.error(f"Error saving session to {file_path}: {e}")
                Console.write_stderr(f"Failed to save session to {file_path}: {e}")

        try:
            while (user_input := session.prompt()) != "exit":
                Console.write_stdout(f"- User: {user_input}")
                new_message = {
                    "role": "user",
                    "content": user_input
                }
                messages.append(new_message)

                # Save session after user input if specified
                if save_session_file:
                    save_session(messages, save_session_file)

                result = chat_session.stream_answer(messages)
                answer = ""
                Console.write_stdout("- Agent: ")
                for chunk in result:
                    answer += chunk
                    Console.write_stdout(f"{chunk}", end="")
                    sys.stdout.flush()
                Console.write_stdout()

                new_answer = {
                    "role": "assistant",
                    "content": answer
                }
                messages.append(new_answer)

                # Save session after agent response if specified
                if save_session_file:
                    save_session(messages, save_session_file)

        except KeyboardInterrupt:
            print("\nExiting chat...")
            # Final save before exit if specified
            if save_session_file:
                save_session(messages, save_session_file)
            sys.exit(0)
        except WrongArgumentError as e:
            Console.write_stderr(f"There was an error: {e}")
        except Exception as e:
            logger.error(f"Error chatting with Agent: {e}")
            Console.write_stderr("An unexpected error has occurred. Please contact the developers.")
            # Save session on error if specified
            if save_session_file:
                save_session(messages, save_session_file)


chat_with_agent_options = [
    Option(
        "agent_name",
        ["--agent-name", "--name", "-n"],
        "You can use the internal name, public name and agent id in order to chat interactively with any agent",
        True
    ),
    Option(
        "gui",
        ["--gui", "-g"],
        "Launch a Streamlit GUI chat interface. No need to specify argument next to the option",
        False
    ),
    Option(
        "save_session",
        ["--save-session", "--ss"],
        "Save the conversation to a JSON file. Provide the file path as the argument.",
        True
    ),
    Option(
        "restore_session",
        ["--restore-session", "--rs"],
        "Restore a conversation from a JSON file. Provide the file path as the argument.",
        True
    ),
]


def get_generate_image(option_list: list):
    model = None
    prompt = None
    n = None
    quality = None
    size = None
    aspect_ratio = None

    for option_flag, option_arg in option_list:
        if option_flag.name == "model":
            model = option_arg
        if option_flag.name == "prompt":
            prompt = option_arg
        if option_flag.name == "n":
            try:
                n = int(option_arg)
                if n < 1 or n > 10:
                    raise WrongArgumentError("n must be an integer between 1 and 10.")
            except ValueError:
                raise WrongArgumentError("n must be a valid integer.")
        if option_flag.name == "quality":
            quality = option_arg
        if option_flag.name == "size":
            size = option_arg
        if option_flag.name == "aspect_ratio":
            aspect_ratio = option_arg

    if not (model and prompt and n is not None and quality and size):
        raise MissingRequirementException("Cannot generate image without specifying model, prompt, n, quality, and size.")

    client = ChatClient()
    try:
        result = client.generate_image(
            model=model,
            prompt=prompt,
            n=n,
            quality=quality,
            size=size,
            aspect_ratio=aspect_ratio
        )
        Console.write_stdout(f"Image generation result: \n{result}\n")
    except Exception as e:
        logger.error(f"Error generating image: {e}")
        Console.write_stderr(f"Failed to generate image: {e}")


generate_image_options = [
    Option(
        "model",
        ["--model", "-m"],
        "The model specification for image generation, e.g., 'openai/gpt-image-1'.",
        True
    ),
    Option(
        "prompt",
        ["--prompt", "-p"],
        "Description of the desired image.",
        True
    ),
    Option(
        "n",
        ["--n"],
        "Number of images to generate (1-10, depending on the model).",
        True
    ),
    Option(
        "quality",
        ["--quality", "-q"],
        "Rendering quality, e.g., 'high'.",
        True
    ),
    Option(
        "size",
        ["--size", "-s"],
        "Image dimensions, e.g., '1024x1024'.",
        True
    ),
    Option(
        "aspect_ratio",
        ["--aspect-ratio", "-ar"],
        "Relationship between image’s width and height, e.g., '1:1', '9:16', '16:9', '3:4', '4:3'.",
        True
    ),
]


def get_edit_image(option_list: list):
    model = None
    prompt = None
    image = None
    size = None
    n = 1
    quality = None

    for option_flag, option_arg in option_list:
        if option_flag.name == "model":
            model = option_arg
        if option_flag.name == "prompt":
            prompt = option_arg
        if option_flag.name == "image":
            image = option_arg
        if option_flag.name == "size":
            size = option_arg
        if option_flag.name == "n":
            try:
                n = int(option_arg)
                if n < 1 or n > 10:
                    raise WrongArgumentError("n must be an integer between 1 and 10.")
            except ValueError:
                raise WrongArgumentError("n must be a valid integer.")
        if option_flag.name == "quality":
            quality = option_arg

    if not (model and prompt and image and size):
        raise MissingRequirementException("Cannot edit image without specifying model, prompt, image, and size.")

    client = ChatClient()
    try:
        result = client.edit_image(
            model=model,
            prompt=prompt,
            image=image,
            size=size,
            n=n,
            quality=quality
        )
        Console.write_stdout(f"Image editing result: \n{result}\n")
    except Exception as e:
        logger.error(f"Error editing image: {e}")
        Console.write_stderr(f"Failed to edit image: {e}")


edit_image_options = [
    Option(
        "model",
        ["--model", "-m"],
        "The model specification for image editing, e.g., 'openai/gpt-image-1'.",
        True
    ),
    Option(
        "prompt",
        ["--prompt", "-p"],
        "Description of the desired edit, e.g., 'remove the ball'.",
        True
    ),
    Option(
        "image",
        ["--image", "-img"],
        "URL of the image to be edited, e.g., 'https://example.com/image.jpg'.",
        True
    ),
    Option(
        "size",
        ["--size", "-s"],
        "Desired dimensions of the output image in pixels, e.g., '1024x1024'.",
        True
    ),
    Option(
        "n",
        ["--n"],
        "Number of edited images to generate (1-10, depending on the model). Default is 1.",
        True
    ),
    Option(
        "quality",
        ["--quality", "-q"],
        "Rendering quality, e.g., 'high', 'medium', 'low'.",
        True
    ),
]


def get_response(option_list: list):
    model = None
    input_text = None
    files = None
    tools = None
    tool_choice = None
    temperature = None
    max_output_tokens = None
    top_p = None
    metadata = None
    user = None
    instructions = None
    reasoning = None
    truncation = None
    parallel_tool_calls = None
    store = None
    stream = False

    for option_flag, option_arg in option_list:
        if option_flag.name == "model":
            model = option_arg
        if option_flag.name == "input":
            input_text = option_arg
        if option_flag.name == "files":
            try:
                files = json.loads(option_arg) if option_arg else None
                if files and not isinstance(files, list):
                    raise WrongArgumentError("files must be a JSON array of file paths")
            except json.JSONDecodeError:
                raise WrongArgumentError("files must be a valid JSON array")
        if option_flag.name == "tools":
            try:
                tools = json.loads(option_arg) if option_arg else None
            except json.JSONDecodeError:
                raise WrongArgumentError("tools must be a valid JSON array")
        if option_flag.name == "tool_choice":
            try:
                tool_choice = json.loads(option_arg) if option_arg else None
            except json.JSONDecodeError:
                tool_choice = option_arg
        if option_flag.name == "temperature":
            temperature = float(option_arg) if option_arg is not None else None
        if option_flag.name == "max_output_tokens":
            max_output_tokens = int(option_arg) if option_arg is not None else None
        if option_flag.name == "top_p":
            top_p = float(option_arg) if option_arg else None
        if option_flag.name == "metadata":
            try:
                metadata = json.loads(option_arg) if option_arg else None
            except json.JSONDecodeError:
                raise WrongArgumentError("metadata must be a valid JSON object")
        if option_flag.name == "user":
            user = option_arg
        if option_flag.name == "instructions":
            instructions = option_arg
        if option_flag.name == "reasoning":
            try:
                reasoning = json.loads(option_arg) if option_arg else None
            except json.JSONDecodeError:
                raise WrongArgumentError("reasoning must be a valid JSON object")
        if option_flag.name == "truncation":
            truncation = option_arg
        if option_flag.name == "parallel_tool_calls":
            parallel_tool_calls = get_boolean_value(option_arg) if option_arg else None
        if option_flag.name == "store":
            store = get_boolean_value(option_arg) if option_arg else None
        if option_flag.name == "stream":
            if option_arg:
                stream = get_boolean_value(option_arg)

    if not (model and input_text):
        raise MissingRequirementException("Cannot get response without specifying model and input")

    client = ChatClient()
    try:
        result = client.get_response(
            model=model,
            input=input_text,
            files=files,
            tools=tools,
            tool_choice=tool_choice,
            temperature=temperature,
            max_output_tokens=max_output_tokens,
            top_p=top_p,
            metadata=metadata,
            user=user,
            instructions=instructions,
            reasoning=reasoning,
            truncation=truncation,
            parallel_tool_calls=parallel_tool_calls,
            store=store,
            stream=stream
        )
        if stream:
            Console.write_stdout("Streaming response:")
            for chunk in result:
                Console.write_stdout(f"{chunk}", end="")
                sys.stdout.flush()
            Console.write_stdout()
        else:
            Console.write_stdout(f"Response result: \n{json.dumps(result, indent=2)}\n")
    except Exception as e:
        logger.error(f"Error getting response: {e}")
        Console.write_stderr(f"Failed to get response: {e}")


response_options = [
    Option(
        "model",
        ["--model", "-m"],
        "The model specification, e.g., 'openai/o1-pro'.",
        True
    ),
    Option(
        "input",
        ["--input", "-i"],
        "The user input text.",
        True
    ),
    Option(
        "files",
        ["--files", "-f"],
        "JSON array of file paths (images or PDFs) to include in the request, e.g., '[\"image.jpg\", \"doc.pdf\"]'.",
        True
    ),
    Option(
        "tools",
        ["--tools"],
        "Optional JSON array of tools (e.g., functions) the model may call.",
        True
    ),
    Option(
        "tool_choice",
        ["--tool-choice"],
        "Optional string (e.g., \"none\", \"auto\") or JSON object to control which tool is called.",
        True
    ),
    Option(
        "temperature",
        ["--temperature", "--temp"],
        "Float value to set randomness of the response (between 0 and 2).",
        True
    ),
    Option(
        "max_output_tokens",
        ["--max-output-tokens"],
        "Integer value to set max tokens in the output.",
        True
    ),
    Option(
        "top_p",
        ["--top-p"],
        "Optional float value for nucleus sampling (between 0 and 1).",
        True
    ),
    Option(
        "metadata",
        ["--metadata"],
        "Optional JSON object with up to 16 key-value pairs to attach to the object.",
        True
    ),
    Option(
        "user",
        ["--user"],
        "Optional string identifier for the end-user.",
        True
    ),
    Option(
        "instructions",
        ["--instructions"],
        "Optional additional instructions for the model.",
        True
    ),
    Option(
        "reasoning",
        ["--reasoning"],
        "Optional JSON object for reasoning configuration, e.g., {\"effort\": \"medium\"}.",
        True
    ),
    Option(
        "truncation",
        ["--truncation"],
        "Optional truncation strategy, e.g., \"disabled\".",
        True
    ),
    Option(
        "parallel_tool_calls",
        ["--parallel-tool-calls"],
        "Optional boolean to enable parallel tool calls. Possible values: 0: OFF; 1: ON",
        True
    ),
    Option(
        "store",
        ["--store"],
        "Optional boolean to store the output. Possible values: 0: OFF; 1: ON",
        True
    ),
    Option(
        "stream",
        ["--stream"],
        "Whether to stream the response. Possible values: 0: OFF; 1: ON",
        True
    ),
]

chat_commands = [
    Command(
        "help",
        ["help", "h"],
        "Display help text",
        show_help,
        ArgumentsEnum.NOT_AVAILABLE,
        [],
        []
    ),
    Command(
        "completion",
        ["completion", "comp"],
        "Get chat completion",
        get_chat_completion,
        ArgumentsEnum.REQUIRED,
        [],
        chat_completion_options
    ),
    Command(
        "iris",
        ["iris"],
        "Interactive chat with Iris",
        chat_with_iris,
        ArgumentsEnum.NOT_AVAILABLE,
        [],
        []
    ),
    Command(
        "agent",
        ["agent"],
        "Interactive chat with Agent",
        chat_with_agent,
        ArgumentsEnum.REQUIRED,
        [],
        chat_with_agent_options
    ),
    Command(
        "generate_image",
        ["generate-image", "gen-img"],
        "Generate an image using the specified model and parameters",
        get_generate_image,
        ArgumentsEnum.REQUIRED,
        [],
        generate_image_options
    ),
    Command(
        "edit_image",
        ["edit-image", "edit-img"],
        "Edit an existing image using the specified model and parameters",
        get_edit_image,
        ArgumentsEnum.REQUIRED,
        [],
        edit_image_options
    ),
    Command(
        "response",
        ["response", "resp"],
        "Get a response using the Responses API with support for images and PDFs",
        get_response,
        ArgumentsEnum.REQUIRED,
        [],
        response_options
    ),

]



from pygeai.evaluation.dataset.clients import EvaluationDatasetClient

client = EvaluationDatasetClient()

dataset_id = "your-dataset-id-here"

response = client.update_dataset(
    dataset_id=dataset_id,
    dataset_name="Updated Dataset Name",
    dataset_description="Updated description",
    dataset_type="E",
    dataset_active=False
)

print(f"Updated dataset: {response}")



import unittest
from unittest.mock import patch, MagicMock
from pygeai.core.embeddings.managers import EmbeddingsManager
from pygeai.core.embeddings.models import EmbeddingConfiguration
from pygeai.core.embeddings.responses import EmbeddingResponse
from pygeai.core.common.exceptions import APIError


class TestEmbeddingsManager(unittest.TestCase):
    """
    python -m unittest pygeai.tests.core.embeddings.test_managers.TestEmbeddingsManager
    """

    def setUp(self):
        self.manager = EmbeddingsManager()
        self.config = EmbeddingConfiguration(
            inputs=["test input"],
            model="test-model"
        )

    @patch('pygeai.core.embeddings.clients.EmbeddingsClient.generate_embeddings')
    def test_generate_embeddings_success(self, mock_generate):
        mock_generate.return_value = {
            "model": "test-model",
            "object": "list",
            "data": [
                {
                    "index": 0,
                    "embedding": [0.1, 0.2, 0.3],
                    "object": "embedding"
                }
            ],
            "usage": {
                "prompt_tokens": 5,
                "total_tokens": 5,
                "total_cost": 0.0001,
                "currency": "USD",
                "prompt_cost": 0.0001,
                "completion_tokens_details": None,
                "prompt_tokens_details": None
            }
        }

        result = self.manager.generate_embeddings(self.config)

        self.assertIsInstance(result, EmbeddingResponse)
        self.assertEqual(result.model, "test-model")
        self.assertEqual(len(result.data), 1)
        self.assertEqual(result.data[0].index, 0)
        self.assertEqual(result.usage.prompt_tokens, 5)
        self.assertEqual(result.usage.total_cost, 0.0001)

    @patch('pygeai.core.embeddings.clients.EmbeddingsClient.generate_embeddings')
    def test_generate_embeddings_with_all_config_parameters(self, mock_generate):
        mock_generate.return_value = {
            "model": "test-model",
            "object": "list",
            "data": [{"index": 0, "embedding": [0.1], "object": "embedding"}],
            "usage": {
                "prompt_tokens": 5,
                "total_tokens": 5,
                "total_cost": 0.0001,
                "currency": "USD",
                "prompt_cost": 0.0001,
                "completion_tokens_details": None,
                "prompt_tokens_details": None
            }
        }

        config = EmbeddingConfiguration(
            inputs=["test"],
            model="test-model",
            encoding_format="base64",
            dimensions=512,
            user="user123",
            input_type="query",
            timeout=30,
            cache=True
        )

        result = self.manager.generate_embeddings(config)

        mock_generate.assert_called_once_with(
            input_list=["test"],
            model="test-model",
            encoding_format="base64",
            dimensions=512,
            user="user123",
            input_type="query",
            timeout=30,
            cache=True
        )
        self.assertIsInstance(result, EmbeddingResponse)

    @patch('pygeai.core.embeddings.clients.EmbeddingsClient.generate_embeddings')
    def test_generate_embeddings_with_error(self, mock_generate):
        mock_generate.return_value = {
            "error": {
                "message": "Invalid model",
                "type": "invalid_request_error"
            }
        }

        with self.assertRaises(APIError) as context:
            self.manager.generate_embeddings(self.config)

        self.assertIn("Error received while generating embeddings", str(context.exception))

    @patch('pygeai.core.embeddings.clients.EmbeddingsClient.generate_embeddings')
    def test_generate_embeddings_multiple_inputs(self, mock_generate):
        mock_generate.return_value = {
            "model": "test-model",
            "object": "list",
            "data": [
                {"index": 0, "embedding": [0.1, 0.2], "object": "embedding"},
                {"index": 1, "embedding": [0.3, 0.4], "object": "embedding"}
            ],
            "usage": {
                "prompt_tokens": 10,
                "total_tokens": 10,
                "total_cost": 0.0002,
                "currency": "USD",
                "prompt_cost": 0.0002,
                "completion_tokens_details": None,
                "prompt_tokens_details": None
            }
        }

        config = EmbeddingConfiguration(
            inputs=["input1", "input2"],
            model="test-model"
        )

        result = self.manager.generate_embeddings(config)

        self.assertEqual(len(result.data), 2)
        self.assertEqual(result.data[0].index, 0)
        self.assertEqual(result.data[1].index, 1)

    @patch('pygeai.core.embeddings.clients.EmbeddingsClient.generate_embeddings')
    def test_generate_embeddings_with_token_details(self, mock_generate):
        mock_generate.return_value = {
            "model": "test-model",
            "object": "list",
            "data": [{"index": 0, "embedding": [0.1], "object": "embedding"}],
            "usage": {
                "prompt_tokens": 5,
                "total_tokens": 5,
                "total_cost": 0.0001,
                "currency": "USD",
                "prompt_cost": 0.0001,
                "completion_tokens_details": {
                    "reasoning_tokens": 0,
                    "cached_tokens": 0
                },
                "prompt_tokens_details": {
                    "reasoning_tokens": 0,
                    "cached_tokens": 5
                }
            }
        }

        result = self.manager.generate_embeddings(self.config)

        self.assertIsNotNone(result.usage.completion_tokens_details)
        self.assertIsNotNone(result.usage.prompt_tokens_details)
        self.assertEqual(result.usage.prompt_tokens_details.cached_tokens, 5)


if __name__ == '__main__':
    unittest.main()



from unittest import TestCase
from pygeai.lab.managers import AILabManager
from pygeai.lab.models import ToolList, FilterSettings
import copy

ai_lab_manager: AILabManager

class TestAILabListToolsIntegration(TestCase):    

    def setUp(self):
        self.ai_lab_manager = AILabManager()
        self.filter_settings = FilterSettings(
            allow_external=False,
            allow_drafts=True,
            access_scope="private"
        )


    def __list_tools(self, filter_settings: FilterSettings = None):
        filter_settings = filter_settings if filter_settings != None else self.filter_settings
        return self.ai_lab_manager.list_tools(filter_settings=filter_settings)
    
    
    def test_private_list_tools(self):
        result = self.__list_tools()
        self.assertIsInstance(result, ToolList , "Expected a list of tools") 

        for tool in result.tools:
            self.assertTrue(tool.access_scope == "private", "Expected all tools to be private")


    def test_public_list_tools(self):
        filter_settings = copy.deepcopy(self.filter_settings)
        filter_settings.access_scope = "public"

        result = self.__list_tools(filter_settings=filter_settings)
        self.assertIsInstance(result, ToolList , "Expected a list of tools") 

        for tool in result.tools:
            self.assertTrue(tool.access_scope == "public", "Expected all tools to be public")

     
    def test_list_tools_small_count(self):
        filter_settings = copy.deepcopy(self.filter_settings)
        filter_settings.count = 2

        result = self.__list_tools(filter_settings=filter_settings)
        self.assertIsInstance(result, ToolList , "Expected a list of tools") 
        
        self.assertEqual(len(result), 2, "Expected list of tools returned to be 2")  


    def test_list_tools_big_count(self):
        filter_settings = copy.deepcopy(self.filter_settings)
        filter_settings.count = 500

        result = self.__list_tools(filter_settings=filter_settings)
        self.assertIsInstance(result, ToolList , "Expected a list of tools") 
        
        self.assertLessEqual(len(result), 500, "Expected list of tools returned to be 500 or less")


    def test_list_tools_allowing_draft(self):
        result = self.__list_tools()
        validated = any(tool.is_draft == True for tool in result.tools)        
        self.assertTrue(
            validated,
            "Expected at least one tool to be a draft"
        )

    
    def test_list_tools_no_draft(self):
        filter_settings = copy.deepcopy(self.filter_settings)
        filter_settings.allow_drafts = False
        result = self.__list_tools(filter_settings=filter_settings)
        
        validated = any(tool.is_draft == True for tool in result.tools)        
        self.assertFalse(
            validated,
            "Expected no draft tools in the list"
        )


    def test_list_tools_invalid_scope(self):
        filter_settings = copy.deepcopy(self.filter_settings)
        filter_settings.scope = "project"
        
        with self.assertRaises(ValueError) as exception:
           self.__list_tools(filter_settings=filter_settings)

        self.assertIn(
            "Scope must be one of builtin, external, api, proxied.",
            str(exception.exception),                    
            f"The expected error about invalid scope was not returned"
        )


    def test_list_tools_allowing_external(self):
        filter_settings = copy.deepcopy(self.filter_settings)
        filter_settings.allow_external = False
        
        result = self.__list_tools(filter_settings=filter_settings)
        self.assertIsInstance(result, ToolList , "Expected a list of tools") 
        





from pygeai.analytics.managers import AnalyticsManager

manager = AnalyticsManager()

response = manager.get_top_10_agents_by_requests(
    start_date="2025-01-01",
    end_date="2026-01-31"
)

print("Top 10 agents by requests:")
for idx, agent in enumerate(response.topAgents, 1):
    print(f"{idx}. {agent.agentName}: {agent.totalRequests} requests")



import unittest
from unittest.mock import patch, MagicMock, mock_open
import json
from pathlib import Path
from pygeai.core.base.session import Session

from pygeai.evaluation.dataset.clients import EvaluationDatasetClient


class TestEvaluationDatasetClient(unittest.TestCase):
    """
    python -m unittest pygeai.tests.evaluation.dataset.test_clients.TestEvaluationDatasetClient
    """

    def setUp(self):
        self.client = EvaluationDatasetClient(api_key="test-key", base_url="http://test.com", eval_url="http://eval.com")
        self.mock_response = MagicMock()
        self.mock_response.json.return_value = {"id": "dataset-123", "status": "success"}
        self.mock_response.status_code = 200

    @patch('pygeai.core.services.rest.ApiService.get')
    def test_list_datasets(self, mock_get):
        mock_get.return_value = self.mock_response
        result = self.client.list_datasets()
        mock_get.assert_called_once()
        self.assertEqual(result, {"id": "dataset-123", "status": "success"})

    @patch('pygeai.core.services.rest.ApiService.post')
    def test_create_dataset_minimal(self, mock_post):
        mock_post.return_value = self.mock_response
        result = self.client.create_dataset(
            dataset_name="Test Dataset",
            dataset_description="Test Description",
            dataset_type="T"
        )
        mock_post.assert_called_once()
        call_args = mock_post.call_args
        self.assertEqual(call_args[1]['data']['dataSetName'], "Test Dataset")
        self.assertEqual(call_args[1]['data']['dataSetDescription'], "Test Description")
        self.assertEqual(call_args[1]['data']['dataSetType'], "T")
        self.assertTrue(call_args[1]['data']['dataSetActive'])

    @patch('pygeai.core.services.rest.ApiService.post')
    def test_create_dataset_with_rows(self, mock_post):
        mock_post.return_value = self.mock_response
        rows = [{"dataSetRowInput": "input1", "dataSetRowExpectedAnswer": "answer1"}]
        result = self.client.create_dataset(
            dataset_name="Test Dataset",
            dataset_description="Test Description",
            dataset_type="T",
            dataset_active=False,
            rows=rows
        )
        mock_post.assert_called_once()
        call_args = mock_post.call_args
        self.assertEqual(call_args[1]['data']['rows'], rows)
        self.assertFalse(call_args[1]['data']['dataSetActive'])

    @patch('pathlib.Path.is_file')
    @patch('pathlib.Path.open')
    @patch('pygeai.core.services.rest.ApiService.post_file_binary')
    def test_create_dataset_from_file_success(self, mock_post, mock_open_file, mock_is_file):
        mock_is_file.return_value = True
        mock_file = MagicMock()
        mock_open_file.return_value = mock_file
        mock_post.return_value = self.mock_response

        result = self.client.create_dataset_from_file("/path/to/dataset.json")
        
        mock_is_file.assert_called_once()
        mock_open_file.assert_called_once_with("rb")
        mock_post.assert_called_once()
        mock_file.close.assert_called_once()
        self.assertEqual(result, {"id": "dataset-123", "status": "success"})

    @patch('pathlib.Path.is_file')
    def test_create_dataset_from_file_not_found(self, mock_is_file):
        mock_is_file.return_value = False
        with self.assertRaises(FileNotFoundError):
            self.client.create_dataset_from_file("/path/to/nonexistent.json")

    @patch('pygeai.core.services.rest.ApiService.get')
    def test_get_dataset(self, mock_get):
        mock_get.return_value = self.mock_response
        result = self.client.get_dataset("dataset-123")
        mock_get.assert_called_once()
        self.assertEqual(result, {"id": "dataset-123", "status": "success"})

    @patch('pygeai.core.services.rest.ApiService.put')
    def test_update_dataset(self, mock_put):
        mock_put.return_value = self.mock_response
        rows = [{"dataSetRowInput": "updated"}]
        result = self.client.update_dataset(
            dataset_id="dataset-123",
            dataset_name="Updated Dataset",
            dataset_description="Updated Description",
            dataset_type="E",
            dataset_active=False,
            rows=rows
        )
        mock_put.assert_called_once()
        call_args = mock_put.call_args
        self.assertEqual(call_args[1]['data']['dataSetName'], "Updated Dataset")
        self.assertEqual(call_args[1]['data']['rows'], rows)

    @patch('pygeai.core.services.rest.ApiService.delete')
    def test_delete_dataset(self, mock_delete):
        mock_delete.return_value = self.mock_response
        result = self.client.delete_dataset("dataset-123")
        mock_delete.assert_called_once()
        self.assertEqual(result, {"id": "dataset-123", "status": "success"})

    @patch('pygeai.core.services.rest.ApiService.post')
    def test_create_dataset_row(self, mock_post):
        mock_post.return_value = self.mock_response
        row = {
            "dataSetRowInput": "input",
            "dataSetRowExpectedAnswer": "answer",
            "dataSetRowContextDocument": "context"
        }
        result = self.client.create_dataset_row("dataset-123", row)
        mock_post.assert_called_once()
        call_args = mock_post.call_args
        self.assertEqual(call_args[1]['data'], row)

    @patch('pygeai.core.services.rest.ApiService.get')
    def test_list_dataset_rows(self, mock_get):
        mock_get.return_value = self.mock_response
        result = self.client.list_dataset_rows("dataset-123")
        mock_get.assert_called_once()
        self.assertEqual(result, {"id": "dataset-123", "status": "success"})

    @patch('pygeai.core.services.rest.ApiService.get')
    def test_get_dataset_row(self, mock_get):
        mock_get.return_value = self.mock_response
        result = self.client.get_dataset_row("dataset-123", "row-456")
        mock_get.assert_called_once()
        self.assertEqual(result, {"id": "dataset-123", "status": "success"})

    @patch('pygeai.core.services.rest.ApiService.put')
    def test_update_dataset_row(self, mock_put):
        mock_put.return_value = self.mock_response
        row = {"dataSetRowInput": "updated input"}
        result = self.client.update_dataset_row("dataset-123", "row-456", row)
        mock_put.assert_called_once()

    @patch('pygeai.core.services.rest.ApiService.delete')
    def test_delete_dataset_row(self, mock_delete):
        mock_delete.return_value = self.mock_response
        result = self.client.delete_dataset_row("dataset-123", "row-456")
        mock_delete.assert_called_once()

    @patch('pygeai.core.services.rest.ApiService.post')
    def test_create_dataset_row_expected_source(self, mock_post):
        mock_post.return_value = self.mock_response
        result = self.client.create_expected_source(
            dataset_id="dataset-123",
            dataset_row_id="row-456",
            expected_source_name="source1",
            expected_source_value="value1",
            expected_source_extension="txt"
        )
        mock_post.assert_called_once()

    @patch('pygeai.core.services.rest.ApiService.get')
    def test_list_dataset_row_expected_sources(self, mock_get):
        mock_get.return_value = self.mock_response
        result = self.client.list_expected_sources("dataset-123", "row-456")
        mock_get.assert_called_once()

    @patch('pygeai.core.services.rest.ApiService.get')
    def test_get_expected_source(self, mock_get):
        mock_get.return_value = self.mock_response
        result = self.client.get_expected_source("dataset-123", "row-456", "source-789")
        mock_get.assert_called_once()

    @patch('pygeai.core.services.rest.ApiService.put')
    def test_update_expected_source(self, mock_put):
        mock_put.return_value = self.mock_response
        result = self.client.update_expected_source(
            dataset_id="dataset-123",
            dataset_row_id="row-456",
            expected_source_id="source-789",
            expected_source_name="updated source",
            expected_source_value="updated value",
            expected_source_extension="pdf"
        )
        mock_put.assert_called_once()

    @patch('pygeai.core.services.rest.ApiService.delete')
    def test_delete_expected_source(self, mock_delete):
        mock_delete.return_value = self.mock_response
        result = self.client.delete_expected_source("dataset-123", "row-456", "source-789")
        mock_delete.assert_called_once()

    @patch('pygeai.core.services.rest.ApiService.post')
    def test_create_dataset_row_filter_variable(self, mock_post):
        mock_post.return_value = self.mock_response
        result = self.client.create_filter_variable(
            dataset_id="dataset-123",
            dataset_row_id="row-456",
            metadata_type="type1",
            filter_variable_key="key1",
            filter_variable_value="value1",
            filter_variable_operator="eq"
        )
        mock_post.assert_called_once()

    @patch('pygeai.core.services.rest.ApiService.get')
    def test_list_dataset_row_filter_variables(self, mock_get):
        mock_get.return_value = self.mock_response
        result = self.client.list_filter_variables("dataset-123", "row-456")
        mock_get.assert_called_once()

    @patch('pygeai.core.services.rest.ApiService.get')
    def test_get_filter_variable(self, mock_get):
        mock_get.return_value = self.mock_response
        result = self.client.get_filter_variable("dataset-123", "row-456", "var-789")
        mock_get.assert_called_once()

    @patch('pygeai.core.services.rest.ApiService.put')
    def test_update_filter_variable(self, mock_put):
        mock_put.return_value = self.mock_response
        result = self.client.update_filter_variable(
            dataset_id="dataset-123",
            dataset_row_id="row-456",
            filter_variable_id="var-789",
            metadata_type="type2",
            filter_variable_key="updated key",
            filter_variable_value="updated value",
            filter_variable_operator="ne"
        )
        mock_put.assert_called_once()

    @patch('pygeai.core.services.rest.ApiService.delete')
    def test_delete_filter_variable(self, mock_delete):
        mock_delete.return_value = self.mock_response
        result = self.client.delete_filter_variable("dataset-123", "row-456", "var-789")
        mock_delete.assert_called_once()

    @patch('pathlib.Path.is_file')
    @patch('pathlib.Path.open')
    @patch('pygeai.core.services.rest.ApiService.post_file_binary')
    def test_upload_dataset_rows_file_success(self, mock_post, mock_open_file, mock_is_file):
        mock_is_file.return_value = True
        mock_file = MagicMock()
        mock_open_file.return_value = mock_file
        mock_post.return_value = self.mock_response

        result = self.client.upload_dataset_rows_file("dataset-123", "/path/to/rows.json")
        
        mock_is_file.assert_called_once()
        mock_open_file.assert_called_once_with("rb")
        mock_post.assert_called_once()
        mock_file.close.assert_called_once()

    @patch('pathlib.Path.is_file')
    def test_upload_dataset_rows_file_not_found(self, mock_is_file):
        mock_is_file.return_value = False
        with self.assertRaises(FileNotFoundError):
            self.client.upload_dataset_rows_file("dataset-123", "/path/to/nonexistent.json")


if __name__ == '__main__':
    unittest.main()




from pygeai import logger
from pygeai.core.base.clients import BaseClient
from pygeai.core.common.exceptions import MissingRequirementException, InvalidAPIResponseException
from pygeai.core.utils.validators import validate_status_code
from pygeai.core.utils.parsers import parse_json_response
from pygeai.gam.endpoints import GET_ACCESS_TOKEN_V2, GET_USER_INFO_V2, IDP_SIGNIN_V1


class GAMClient(BaseClient):

    def generate_signing_url(
            self,
            client_id: str = None,
            redirect_uri: str = None,
            scope: str = "gam_user_data",
            state: str = None,
            response_type: str = "code"
    ):
        """
        Generates the URL for the GAM OAuth 2.0 signin endpoint to initiate user authentication.
        This method does not perform the signin itself but provides the URL for redirection in a browser-based flow.

        :param client_id: str - Client ID of the application. Required.
        :param redirect_uri: str - Callback URL configured in the application. Required.
        :param scope: str - Scope of the user account to access (e.g., "gam_user_data").
            Defaults to "gam_user_data".
        :param state: str - Random string to store the status before the request. Required.
        :param response_type: str - Response type for the signin request. Defaults to "code".
        :return: str - The URL to redirect to for user authentication in a browser.
        :raises MissingRequirementException: If required parameters are missing.
        """
        if not all([client_id, redirect_uri, state]):
            raise MissingRequirementException("client_id, redirect_uri, and state are required.")

        params = {
            "response_type": response_type,
            "client_id": client_id,
            "redirect_uri": redirect_uri,
            "state": state
        }
        if scope:
            params["scope"] = scope

        endpoint = f"{self.api_service.base_url}{IDP_SIGNIN_V1}"
        redirect_url = f"{endpoint}?{'&'.join(f'{k}={v}' for k, v in params.items())}"
        return redirect_url

    def get_access_token(
            self,
            client_id: str = None,
            client_secret: str = None,
            grant_type: str = "password",
            authentication_type_name: str = "local",
            scope: str = "gam_user_data",
            username: str = None,
            password: str = None,
            initial_properties: dict = None,
            repository: str = None,
            request_token_type: str = "OAuth"
    ):
        """
        Retrieves an access token by sending a POST request to the GAM OAuth 2.0 access token endpoint.

        :param client_id: str - Application Client ID. Required.
        :param client_secret: str - Application Client Secret. Required.
        :param grant_type: str - Grant type for authentication. Must be "password". Defaults to "password".
        :param authentication_type_name: str - Authentication type name. Defaults to "local".
        :param scope: str - Scope of the user account to access (e.g., "gam_user_data+gam_user_roles").
            Defaults to "gam_user_data".
        :param username: str - Username of the user to be authenticated. Required.
        :param password: str - Password of the user to be authenticated. Required.
        :param initial_properties: dict - User custom properties array (e.g., [{"Id":"Company","Value":"GeneXus"}]). Optional.
        :param repository: str - Repository identifier, used only if IDP is multitenant. Optional.
        :param request_token_type: str - Determines the token type to return and security policy.
            Options are "OAuth" or "Web". Defaults to "OAuth".
        :return: dict or str - Access token response containing access_token, token_type, expires_in,
            refresh_token, scope, and user_guid; returns raw text if JSON parsing fails.
        """
        data = {
            "grant_type": grant_type,
            "authentication_type_name": authentication_type_name,
            "scope": scope,
            "request_token_type": request_token_type
        }
        if client_id is not None:
            data["client_id"] = client_id
        if client_secret is not None:
            data["client_secret"] = client_secret
        if username is not None:
            data["username"] = username
        if password is not None:
            data["password"] = password
        if initial_properties is not None:
            data["initial_properties"] = initial_properties
        if repository is not None:
            data["repository"] = repository

        headers = {
            "Content-Type": "application/x-www-form-urlencoded"
        }
        response = self.api_service.post(
            endpoint=GET_ACCESS_TOKEN_V2,
            data=data,
            headers=headers,
            form=True
        )
        validate_status_code(response)
        return parse_json_response(response, "get access token")

    def get_user_info(
            self,
            access_token: str
    ):
        """
        Retrieves user information by sending a GET request to the GAM OAuth 2.0 userinfo endpoint.

        :param access_token: str - Access token obtained from the access_token endpoint. Required.
        :return: dict or str - User information response containing guid, username, email, and other user
            details based on requested scopes; returns raw text if JSON parsing fails.
        """
        headers = {
            "Authorization": access_token,
            "Content-Type": "application/x-www-form-urlencoded"
        }
        response = self.api_service.get(
            endpoint=GET_USER_INFO_V2,
            headers=headers
        )
        validate_status_code(response)
        return parse_json_response(response, "get user info")

    def refresh_access_token(
            self,
            client_id: str = None,
            client_secret: str = None,
            grant_type: str = "refresh_token",
            refresh_token: str = None
    ):
        """
        Refreshes an access token by sending a GET request to the GAM OAuth 2.0 access token endpoint.

        :param client_id: str - Application Client ID. Required.
        :param client_secret: str - Application Client Secret. Required.
        :param grant_type: str - Grant type for authentication. Must be "refresh_token". Defaults to "refresh_token".
        :param refresh_token: str - Refresh token obtained from a previous access token response. Required.
        :return: dict or str - New access token response containing access_token, token_type, expires_in,
            refresh_token, scope, and user_guid; returns raw text if JSON parsing fails.
        """
        data = {
            "grant_type": grant_type,
        }
        if client_id is not None:
            data["client_id"] = client_id
        if client_secret is not None:
            data["client_secret"] = client_secret
        if refresh_token is not None:
            data["refresh_token"] = refresh_token

        headers = {
            "Content-Type": "application/x-www-form-urlencoded"
        }
        response = self.api_service.post(
            endpoint=GET_ACCESS_TOKEN_V2,
            data=data,
            headers=headers,
            form=True
        )
        validate_status_code(response)
        return parse_json_response(response, "refresh access token")

    def get_authentication_types(self):
        response = self.api_service.get(
            endpoint=GET_ACCESS_TOKEN_V2,
            params={},
        )
        validate_status_code(response)
        return parse_json_response(response, "get authentication types")



import os
import sys
from functools import lru_cache
from pathlib import Path
import configparser

from pygeai import logger

HOME_DIR = Path.home()
SETTINGS_DIR = HOME_DIR / ".geai"
SETTINGS_DIR.mkdir(parents=True, exist_ok=True)


class SettingsManager:
    """
    Base class to handle settings.
    If environment variables are defined, it retrieves settings from them first.
    Else, it looks for settings in the .geai/credentials file
    """

    GEAI_SETTINGS_DIR = str(SETTINGS_DIR)
    GEAI_CREDS_FILE = SETTINGS_DIR / "credentials"

    def __init__(self):
        self.config = configparser.ConfigParser()

        if self.GEAI_CREDS_FILE.exists():
            self.config.read(self.GEAI_CREDS_FILE)
        else:
            self.GEAI_CREDS_FILE.touch()
            sys.stdout.write(f"INFO: Credentials file not found. Creating empty one at {self.GEAI_CREDS_FILE}\n")

    def has_value(self, setting_key: str, alias: str):
        """Checks if a setting value exists for a specific alias in the credentials file."""
        return alias in self.config and setting_key in self.config[alias]

    def get_setting_value(self, setting_key: str, alias: str):
        """Reads a setting value for a specific alias from the credentials file."""
        if alias not in self.config:
            logger.warning(f"Alias '{alias}' not found in the credentials file.")
            return
        
        if setting_key not in self.config[alias]:
            sys.stdout.write(f"'{setting_key}' not found in alias '{alias}' in the credentials file. Adding empty value.\n")
            # SET ADDITIONAL VARS
            if setting_key.lower() == "GEAI_API_EVAL_URL".lower():
                self.set_eval_url("", alias)
            if setting_key.lower() == "GEAI_OAUTH_ACCESS_TOKEN".lower():
                self.set_access_token("", alias)
            if setting_key.lower() == "GEAI_PROJECT_ID".lower():
                self.set_project_id("", alias)

        return self.config[alias].get(setting_key, "")

    def set_setting_value(self, setting_key: str, setting_value: str, alias: str):
        """Writes or updates a setting value for a specific alias in the credentials file."""
        if alias not in self.config:
            self.config.add_section(alias)

        self.config[alias][setting_key] = setting_value

        with self.GEAI_CREDS_FILE.open("w") as file:
            self.config.write(file)

    def get_api_key(self, alias: str = "default"):
        api_key = os.environ.get("GEAI_API_KEY") if not alias or alias == "default" else None
        if not api_key:
            api_key = self.get_setting_value("GEAI_API_KEY", alias)

        return api_key

    def set_api_key(self, api_key, alias: str = "default"):
        self.set_setting_value("GEAI_API_KEY", api_key, alias)

    def get_base_url(self, alias: str = "default"):
        base_url = os.environ.get("GEAI_API_BASE_URL") if not alias or alias == "default" else None
        if not base_url:
            base_url = self.get_setting_value("GEAI_API_BASE_URL", alias)

        return base_url

    def set_base_url(self, base_url, alias: str = "default"):
        self.set_setting_value("GEAI_API_BASE_URL", base_url, alias)

    def get_access_token(self, alias: str = "default"):
        access_token = os.environ.get("GEAI_OAUTH_ACCESS_TOKEN") if not alias or alias == "default" else None
        if not access_token:
            access_token = self.get_setting_value("GEAI_OAUTH_ACCESS_TOKEN", alias)

        return access_token

    def set_access_token(self, access_token, alias: str = "default"):
        self.set_setting_value("GEAI_OAUTH_ACCESS_TOKEN", access_token, alias)

    def get_project_id(self, alias: str = "default"):
        project_id = os.environ.get("GEAI_PROJECT_ID") if not alias or alias == "default" else None
        if not project_id:
            project_id = self.get_setting_value("GEAI_PROJECT_ID", alias)

        return project_id

    def set_project_id(self, project_id, alias: str = "default"):
        self.set_setting_value("GEAI_PROJECT_ID", project_id, alias)

    def get_eval_url(self, alias: str = "default"):
        eval_url = os.environ.get("GEAI_API_EVAL_URL") if not alias or alias == "default" else None
        if not eval_url:
            eval_url = self.get_setting_value("GEAI_API_EVAL_URL", alias)

        return eval_url

    def set_eval_url(self, eval_url, alias: str = "default"):
        self.set_setting_value("GEAI_API_EVAL_URL", eval_url, alias)

    def list_aliases(self):
        """Returns a dict of all aliases and their base URLs."""
        return {
            section: self.config[section].get("GEAI_API_BASE_URL", "")
            for section in self.config.sections()
        }

    def remove_alias(self, alias: str):
        """Removes a specific alias and its settings from the credentials file."""
        if alias in self.config:
            self.config.remove_section(alias)
            with self.GEAI_CREDS_FILE.open("w") as file:
                self.config.write(file)
            logger.info(f"Alias '{alias}' removed from the credentials file.")
        else:
            logger.warning(f"Alias '{alias}' not found in the credentials file.")


@lru_cache()
def get_settings():
    return SettingsManager()


if __name__ == "__main__":
    settings = get_settings()
    geai_base_url = settings.get_base_url()
    geai_eval_url = settings.get_eval_url()
    print(f"base_url: {geai_base_url}")
    print(f"eval_url: {geai_eval_url}")



from typing import Union
from uuid import uuid4
from pydantic import ValidationError
import logging
from pygeai.core.common.exceptions import MissingRequirementException
from pygeai.core.models import Assistant
from pygeai.lab.models import Agent, AgentData, Prompt, LlmConfig, Sampling, Model, ModelList
from pygeai.core.base.responses import ErrorListResponse
from pygeai.assistant.managers import AssistantManager
from pygeai.lab.managers import AILabManager

logging.basicConfig(level=logging.INFO)
logger = logging.getLogger(__name__)


def transform_assistant_to_agent(assistant: Assistant) -> Agent:
    """
    Transform an Assistant object into an Agent object, using Assistant.name as Agent.name.
    """
    try:
        if not assistant.name:
            raise ValueError("Assistant.name cannot be None or empty")

        # Map status
        status_map = {1: "active", 2: "inactive"}
        agent_status = status_map.get(assistant.status, "active")

        # Map LLM settings to LlmConfig
        llm_config = None
        if assistant.llm_settings:
            sampling = Sampling(
                temperature=assistant.llm_settings.temperature or 0.7,  # Default from Assistant example
                top_k=50,  # Default, as Agent example uses top_k=0 which may not be suitable
                top_p=assistant.llm_settings.top_p or 1.0
            )
            llm_config = LlmConfig(
                max_tokens=assistant.llm_settings.max_tokens or 1000,  # Default from Assistant example
                timeout=60,  # Default, as Agent example uses 0 which may not be suitable
                sampling=sampling
            )

        # Map revisions to models
        models = []
        if assistant.revisions:
            for revision in assistant.revisions:
                model_prompt = {"instructions": revision.prompt} if revision.prompt else None
                model = Model(
                    name=revision.model_name or f"model_{revision.revision_id or uuid4().hex}",
                    llm_config=llm_config,
                    prompt=model_prompt
                )
                models.append(model)
        else:
            # Use llm_settings.model_name if no revisions, aligning with Assistant example
            if assistant.llm_settings and assistant.llm_settings.model_name:
                models.append(Model(
                    name=str(assistant.llm_settings.model_name),  # e.g., "gpt-4"
                    llm_config=llm_config,
                    prompt=None
                ))
        models = ModelList(models=models) if models else ModelList(models=[])

        # Create Prompt
        prompt = Prompt(
            instructions=assistant.prompt or "Default instructions for the agent",
            inputs=[],  # Assistant doesn't provide inputs, unlike Agent example
            outputs=[],  # Assistant doesn't provide outputs, unlike Agent example
            context=None,
            examples=None
        )

        # Create AgentData
        agent_data = AgentData(
            prompt=prompt,
            llm_config=llm_config or LlmConfig(
                max_tokens=1000,  # Default from Assistant example
                timeout=60,
                sampling=Sampling(temperature=0.7, top_k=50, top_p=1.0)  # Defaults from Assistant example
            ),
            models=models,
            resource_pools=None
        )

        # Use welcome_data.description for job_description if available, else fallback to description
        job_description = (
            assistant.welcome_data.description
            if assistant.welcome_data and assistant.welcome_data.description
            else assistant.description
        )

        # Create Agent
        agent = Agent(
            id=assistant.id or str(uuid4()),
            status=agent_status,
            name=assistant.name,  # Use Assistant.name directly
            access_scope="private",  # Default from Agent example
            public_name=None,  # Agent example uses a specific format, but Assistant doesn't provide this
            avatar_image=None,  # Agent example has a URL, but Assistant doesn't provide this
            description=assistant.description,
            job_description=job_description,
            is_draft=True,  # Default, unlike Agent example's False
            is_readonly=False,
            revision=assistant.revisions[0].revision_id if assistant.revisions else None,
            version=None,
            agent_data=agent_data
        )

        return agent

    except ValidationError as e:
        logger.error(f"Validation error while transforming assistant {assistant.id}: {e}")
        raise
    except ValueError as e:
        logger.error(f"Value error while transforming assistant {assistant.id}: {e}")
        raise
    except Exception as e:
        logger.error(f"Unexpected error while transforming assistant {assistant.id}: {e}")
        raise

def fetch_transform_create_agent(
    assistant_id: str,
    project_id: str,
    detail: str = "full",
    automatic_publish: bool = False
) -> Union[Agent, ErrorListResponse]:
    """
    Fetch an Assistant by ID, transform it to an Agent, and create it in the specified project.

    Args:
        assistant_id (str): The ID of the Assistant to fetch.
        project_id (str): The ID of the project where the Agent will be created.
        detail (str, optional): The detail level for fetching Assistant ("summary" or "full"). Defaults to "full".
        automatic_publish (bool, optional): Whether to automatically publish the created Agent. Defaults to False.

    Returns:
        Union[Agent, ErrorListResponse]: The created Agent or an error response.
    """
    try:
        # Initialize managers
        assistant_manager = AssistantManager()
        ailab_manager = AILabManager()

        # Fetch Assistant
        logger.info(f"Fetching Assistant with ID: {assistant_id}")
        assistant = assistant_manager.get_assistant_data(assistant_id=assistant_id, detail=detail)
        if isinstance(assistant, ErrorListResponse):
            logger.error(f"Failed to fetch Assistant: {assistant}")
            return assistant

        # Transform to Agent
        logger.info(f"Transforming Assistant {assistant_id} to Agent")
        agent = transform_assistant_to_agent(assistant)

        # Create Agent
        logger.info(f"Creating Agent in project {project_id}")
        result = ailab_manager.create_agent(
            project_id=project_id,
            agent=agent,
            automatic_publish=automatic_publish
        )

        logger.info(f"Successfully created Agent with ID: {result.id}, Name: {result.name}")
        return result

    except MissingRequirementException as e:
        logger.error(f"Missing requirement: {e}")
    except ValidationError as e:
        logger.error(f"Validation error: {e}")
    except ValueError as e:
        logger.error(f"Value error: {e}")
    except Exception as e:
        logger.error(f"Unexpected error: {e}")


if __name__ == "__main__":
    try:
        ASSISTANT_ID = "3806a85f-cb06-4724-8dba-da19b0a702eb"
        PROJECT_ID = "1956c032-3c66-4435-acb8-6a06e52f819f"

        result = fetch_transform_create_agent(
            assistant_id=ASSISTANT_ID,
            project_id=PROJECT_ID,
            detail="full",
            automatic_publish=True
        )
        if isinstance(result, Agent):
            print(f"Created Agent: ID={result.id}, Name={result.name}")
        else:
            print(f"Error: {result}")
    except Exception as e:
        print(f"Error in main: {e}")


from pygeai.chat.clients import ChatClient

# Example: Using get_response with reasoning parameter
client = ChatClient()

model = "openai/o1-preview"
input_text = "Explain the concept of quantum entanglement in simple terms"

# Using reasoning parameter to control reasoning effort
response = client.get_response(
    model=model,
    input=input_text,
    reasoning={
        "type": "default",  # Can be "default", "high", "medium", "low"
        "effort": "medium"
    },
    max_output_tokens=2000
)

print("Response with reasoning:")
print(response)



from pygeai.lab.managers import AILabManager
from pygeai.lab.models import KnowledgeBase

manager = AILabManager()

knowledge_base = KnowledgeBase(
    name="sample-kb",
    artifact_type_name=["sample-artifact"],
    # artifacts=["artifact-001", "artifact-002"],
    metadata=["issue_id", "priority"]
)

result = manager.create_knowledge_base(
    knowledge_base=knowledge_base
)

print(f"Created knowledge base: {result.name}, ID: {result.id}")




from pygeai.chat.clients import ChatClient

# Example: Using get_response with store parameter to persist conversation
client = ChatClient()

model = "openai/gpt-4-turbo-preview"
input_text = "Remember that my favorite color is blue. What did I just tell you?"

# Use store parameter to enable conversation storage
response = client.get_response(
    model=model,
    input=input_text,
    store=True,  # Store this conversation for future reference
    user="user_123",  # Associate with specific user
    metadata={
        "session_id": "session_abc",
        "conversation_type": "preference_learning"
    },
    temperature=0.7,
    max_output_tokens=500
)

print("Response with conversation storage:")
print(response)

# Follow-up request can reference stored context
follow_up = client.get_response(
    model=model,
    input="What's my favorite color?",
    store=True,
    user="user_123",
    metadata={
        "session_id": "session_abc"
    }
)

print("\nFollow-up response:")
print(follow_up)



from pygeai.organization.managers import OrganizationManager

manager = OrganizationManager(alias="sdkorg")

response = manager.get_memberships(
    email="alejandro.trinidad@globant.com",
    start_page=1,
    page_size=10,
    #order_key="organizationName",
    #order_direction="asc"
)
print(f"response: {response}")



import asyncio
import logging

from collections.abc import AsyncGenerator
from typing import cast

from a2a.server.agent_execution import (
    AgentExecutor,
    RequestContext,
    RequestContextBuilder,
    SimpleRequestContextBuilder,
)
from a2a.server.context import ServerCallContext
from a2a.server.events import (
    Event,
    EventConsumer,
    EventQueue,
    InMemoryQueueManager,
    QueueManager,
)
from a2a.server.request_handlers.request_handler import RequestHandler
from a2a.server.tasks import (
    PushNotifier,
    ResultAggregator,
    TaskManager,
    TaskStore,
)
from a2a.types import (
    InternalError,
    Message,
    MessageSendConfiguration,
    MessageSendParams,
    PushNotificationConfig,
    Task,
    TaskIdParams,
    TaskNotFoundError,
    TaskPushNotificationConfig,
    TaskQueryParams,
    UnsupportedOperationError,
)
from a2a.utils.errors import ServerError
from a2a.utils.telemetry import SpanKind, trace_class


logger = logging.getLogger(__name__)


@trace_class(kind=SpanKind.SERVER)
class DefaultRequestHandler(RequestHandler):
    """Default request handler for all incoming requests.

    This handler provides default implementations for all A2A JSON-RPC methods,
    coordinating between the `AgentExecutor`, `TaskStore`, `QueueManager`,
    and optional `PushNotifier`.
    """

    _running_agents: dict[str, asyncio.Task]

    def __init__(
        self,
        agent_executor: AgentExecutor,
        task_store: TaskStore,
        queue_manager: QueueManager | None = None,
        push_notifier: PushNotifier | None = None,
        request_context_builder: RequestContextBuilder | None = None,
    ) -> None:
        """Initializes the DefaultRequestHandler.

        Args:
            agent_executor: The `AgentExecutor` instance to run agent logic.
            task_store: The `TaskStore` instance to manage task persistence.
            queue_manager: The `QueueManager` instance to manage event queues. Defaults to `InMemoryQueueManager`.
            push_notifier: The `PushNotifier` instance for sending push notifications. Defaults to None.
            request_context_builder: The `RequestContextBuilder` instance used
              to build request contexts. Defaults to `SimpleRequestContextBuilder`.
        """
        self.agent_executor = agent_executor
        self.task_store = task_store
        self._queue_manager = queue_manager or InMemoryQueueManager()
        self._push_notifier = push_notifier
        self._request_context_builder = (
            request_context_builder
            or SimpleRequestContextBuilder(
                should_populate_referred_tasks=False, task_store=self.task_store
            )
        )
        # TODO: Likely want an interface for managing this, like AgentExecutionManager.
        self._running_agents = {}
        self._running_agents_lock = asyncio.Lock()

    async def on_get_task(
        self,
        params: TaskQueryParams,
        context: ServerCallContext | None = None,
    ) -> Task | None:
        """Default handler for 'tasks/get'."""
        task: Task | None = await self.task_store.get(params.id)
        if not task:
            raise ServerError(error=TaskNotFoundError())
        return task

    async def on_cancel_task(
        self, params: TaskIdParams, context: ServerCallContext | None = None
    ) -> Task | None:
        """Default handler for 'tasks/cancel'.

        Attempts to cancel the task managed by the `AgentExecutor`.
        """
        task: Task | None = await self.task_store.get(params.id)
        if not task:
            raise ServerError(error=TaskNotFoundError())

        task_manager = TaskManager(
            task_id=task.id,
            context_id=task.contextId,
            task_store=self.task_store,
            initial_message=None,
        )
        result_aggregator = ResultAggregator(task_manager)

        queue = await self._queue_manager.tap(task.id)
        if not queue:
            queue = EventQueue()

        await self.agent_executor.cancel(
            RequestContext(
                None,
                task_id=task.id,
                context_id=task.contextId,
                task=task,
            ),
            queue,
        )
        # Cancel the ongoing task, if one exists.
        if producer_task := self._running_agents.get(task.id):
            producer_task.cancel()

        consumer = EventConsumer(queue)
        result = await result_aggregator.consume_all(consumer)
        if isinstance(result, Task):
            return result

        raise ServerError(
            error=InternalError(
                message='Agent did not return valid response for cancel'
            )
        )

    async def _run_event_stream(
        self, request: RequestContext, queue: EventQueue
    ) -> None:
        """Runs the agent's `execute` method and closes the queue afterwards.

        Args:
            request: The request context for the agent.
            queue: The event queue for the agent to publish to.
        """
        await self.agent_executor.execute(request, queue)
        await queue.close()

    async def on_message_send(
        self,
        params: MessageSendParams,
        context: ServerCallContext | None = None,
    ) -> Message | Task:
        """Default handler for 'message/send' interface (non-streaming).

        Starts the agent execution for the message and waits for the final
        result (Task or Message).
        """
        task_manager = TaskManager(
            task_id=params.message.taskId,
            context_id=params.message.contextId,
            task_store=self.task_store,
            initial_message=params.message,
        )
        task: Task | None = await task_manager.get_task()
        if task:
            task = task_manager.update_with_message(params.message, task)
            if self.should_add_push_info(params):
                assert isinstance(self._push_notifier, PushNotifier)
                assert isinstance(
                    params.configuration, MessageSendConfiguration
                )
                assert isinstance(
                    params.configuration.pushNotificationConfig,
                    PushNotificationConfig,
                )
                await self._push_notifier.set_info(
                    task.id, params.configuration.pushNotificationConfig
                )
        request_context = await self._request_context_builder.build(
            params=params,
            task_id=task.id if task else None,
            context_id=params.message.contextId,
            task=task,
            context=context,
        )

        task_id = cast('str', request_context.task_id)
        # Always assign a task ID. We may not actually upgrade to a task, but
        # dictating the task ID at this layer is useful for tracking running
        # agents.
        queue = await self._queue_manager.create_or_tap(task_id)
        result_aggregator = ResultAggregator(task_manager)
        # TODO: to manage the non-blocking flows.
        producer_task = asyncio.create_task(
            self._run_event_stream(
                request_context,
                queue,
            )
        )
        await self._register_producer(task_id, producer_task)

        consumer = EventConsumer(queue)
        producer_task.add_done_callback(consumer.agent_task_callback)

        interrupted = False
        try:
            (
                result,
                interrupted,
            ) = await result_aggregator.consume_and_break_on_interrupt(consumer)
            if not result:
                raise ServerError(error=InternalError())

            if isinstance(result, Task) and task_id != result.id:
                logger.error(
                    f'Agent generated task_id={result.id} does not match the RequestContext task_id={task_id}.'
                )
                raise ServerError(
                    InternalError(message='Task ID mismatch in agent response')
                )

        finally:
            if interrupted:
                # TODO: Track this disconnected cleanup task.
                asyncio.create_task(  # noqa: RUF006
                    self._cleanup_producer(producer_task, task_id)
                )
            else:
                await self._cleanup_producer(producer_task, task_id)

        return result

    async def on_message_send_stream(
        self,
        params: MessageSendParams,
        context: ServerCallContext | None = None,
    ) -> AsyncGenerator[Event]:
        """Default handler for 'message/stream' (streaming).

        Starts the agent execution and yields events as they are produced
        by the agent.
        """
        task_manager = TaskManager(
            task_id=params.message.taskId,
            context_id=params.message.contextId,
            task_store=self.task_store,
            initial_message=params.message,
        )
        task: Task | None = await task_manager.get_task()

        if task:
            task = task_manager.update_with_message(params.message, task)

            if self.should_add_push_info(params):
                assert isinstance(self._push_notifier, PushNotifier)
                assert isinstance(
                    params.configuration, MessageSendConfiguration
                )
                assert isinstance(
                    params.configuration.pushNotificationConfig,
                    PushNotificationConfig,
                )
                await self._push_notifier.set_info(
                    task.id, params.configuration.pushNotificationConfig
                )
        else:
            queue = EventQueue()
        result_aggregator = ResultAggregator(task_manager)
        request_context = await self._request_context_builder.build(
            params=params,
            task_id=task.id if task else None,
            context_id=params.message.contextId,
            task=task,
            context=context,
        )

        task_id = cast('str', request_context.task_id)
        queue = await self._queue_manager.create_or_tap(task_id)
        producer_task = asyncio.create_task(
            self._run_event_stream(
                request_context,
                queue,
            )
        )
        await self._register_producer(task_id, producer_task)

        try:
            consumer = EventConsumer(queue)
            producer_task.add_done_callback(consumer.agent_task_callback)
            async for event in result_aggregator.consume_and_emit(consumer):
                if isinstance(event, Task):
                    if task_id != event.id:
                        logger.error(
                            f'Agent generated task_id={event.id} does not match the RequestContext task_id={task_id}.'
                        )
                        raise ServerError(
                            InternalError(
                                message='Task ID mismatch in agent response'
                            )
                        )

                    if (
                        self._push_notifier
                        and params.configuration
                        and params.configuration.pushNotificationConfig
                    ):
                        await self._push_notifier.set_info(
                            task_id,
                            params.configuration.pushNotificationConfig,
                        )

                if self._push_notifier and task_id:
                    latest_task = await result_aggregator.current_result
                    if isinstance(latest_task, Task):
                        await self._push_notifier.send_notification(latest_task)
                yield event
        finally:
            await self._cleanup_producer(producer_task, task_id)

    async def _register_producer(
        self, task_id: str, producer_task: asyncio.Task
    ) -> None:
        """Registers the agent execution task with the handler."""
        async with self._running_agents_lock:
            self._running_agents[task_id] = producer_task

    async def _cleanup_producer(
        self,
        producer_task: asyncio.Task,
        task_id: str,
    ) -> None:
        """Cleans up the agent execution task and queue manager entry."""
        await producer_task
        await self._queue_manager.close(task_id)
        async with self._running_agents_lock:
            self._running_agents.pop(task_id, None)

    async def on_set_task_push_notification_config(
        self,
        params: TaskPushNotificationConfig,
        context: ServerCallContext | None = None,
    ) -> TaskPushNotificationConfig:
        """Default handler for 'tasks/pushNotificationConfig/set'.

        Requires a `PushNotifier` to be configured.
        """
        if not self._push_notifier:
            raise ServerError(error=UnsupportedOperationError())

        task: Task | None = await self.task_store.get(params.taskId)
        if not task:
            raise ServerError(error=TaskNotFoundError())

        await self._push_notifier.set_info(
            params.taskId,
            params.pushNotificationConfig,
        )

        return params

    async def on_get_task_push_notification_config(
        self,
        params: TaskIdParams,
        context: ServerCallContext | None = None,
    ) -> TaskPushNotificationConfig:
        """Default handler for 'tasks/pushNotificationConfig/get'.

        Requires a `PushNotifier` to be configured.
        """
        if not self._push_notifier:
            raise ServerError(error=UnsupportedOperationError())

        task: Task | None = await self.task_store.get(params.id)
        if not task:
            raise ServerError(error=TaskNotFoundError())

        push_notification_config = await self._push_notifier.get_info(params.id)
        if not push_notification_config:
            raise ServerError(error=InternalError())

        return TaskPushNotificationConfig(
            taskId=params.id, pushNotificationConfig=push_notification_config
        )

    async def on_resubscribe_to_task(
        self,
        params: TaskIdParams,
        context: ServerCallContext | None = None,
    ) -> AsyncGenerator[Event]:
        """Default handler for 'tasks/resubscribe'.

        Allows a client to re-attach to a running streaming task's event stream.
        Requires the task and its queue to still be active.
        """
        task: Task | None = await self.task_store.get(params.id)
        if not task:
            raise ServerError(error=TaskNotFoundError())

        task_manager = TaskManager(
            task_id=task.id,
            context_id=task.contextId,
            task_store=self.task_store,
            initial_message=None,
        )

        result_aggregator = ResultAggregator(task_manager)

        queue = await self._queue_manager.tap(task.id)
        if not queue:
            raise ServerError(error=TaskNotFoundError())

        consumer = EventConsumer(queue)
        async for event in result_aggregator.consume_and_emit(consumer):
            yield event

    def should_add_push_info(self, params: MessageSendParams) -> bool:
        """Determines if push notification info should be set for a task."""
        return bool(
            self._push_notifier
            and params.configuration
            and params.configuration.pushNotificationConfig
        )




from pygeai import logger
from pygeai.core.common.exceptions import InvalidAPIResponseException
from pygeai.core.utils.validators import validate_status_code
from pygeai.core.utils.parsers import parse_json_response
from pygeai.lab.clients import AILabClient
from pygeai.lab.strategies.endpoints import LIST_REASONING_STRATEGIES_V2, CREATE_REASONING_STRATEGY_V2, \
    UPDATE_REASONING_STRATEGY_V2, UPSERT_REASONING_STRATEGY_V2, GET_REASONING_STRATEGY_V2


class ReasoningStrategyClient(AILabClient):

    def list_reasoning_strategies(
            self,
            name: str = "",
            start: str = "0",
            count: str = "100",
            allow_external: bool = True,
            access_scope: str = "public"
    ):
        """
        Retrieves a list of reasoning strategies filtered by specified criteria.

        :param name: str, optional - Name to filter strategies (default: "").
        :param start: str, optional - Starting index for pagination (default: "0").
        :param count: str, optional - Number of strategies to retrieve (default: "100").
        :param allow_external: bool, optional - Include external strategies (default: True).
        :param access_scope: str, optional - Access scope, "public" or "private" (default: "public").
        :return: dict - List of reasoning strategies.
        :raises ValueError: If access_scope is not "public" or "private".
        :raises InvalidAPIResponseException: If the API response cannot be parsed.
        """
        valid_access_scopes = ["public", "private"]
        if access_scope not in valid_access_scopes:
            raise ValueError("Access scope must be either 'public' or 'private'.")

        endpoint = LIST_REASONING_STRATEGIES_V2
        headers = {"Authorization": self.api_service.token}
        params = {
            "name": name,
            "start": start,
            "count": count,
            "allowExternal": allow_external,
            "accessScope": access_scope,
        }

        logger.debug("Listing reasoning strategies")

        response = self.api_service.get(endpoint=endpoint, headers=headers, params=params)
        validate_status_code(response)
        return parse_json_response(response, "list reasoning strategies")


    def create_reasoning_strategy(
            self,
            name: str,
            system_prompt: str,
            access_scope: str = "public",
            strategy_type: str = "addendum",
            localized_descriptions: list = None,
            automatic_publish: bool = False
    ):
        """
        Creates a new reasoning strategy in the specified project.

        :param name: str - Name of the reasoning strategy.
        :param system_prompt: str - System prompt for the strategy.
        :param access_scope: str, optional - Access scope, "public" or "private" (default: "public").
        :param strategy_type: str, optional - Strategy type, e.g., "addendum" (default: "addendum").
        :param localized_descriptions: list, optional - List of localized description dictionaries.
        :param automatic_publish: bool, optional - Publish strategy after creation (default: False).
        :return: dict - Created strategy details.
        :raises InvalidAPIResponseException: If the API response cannot be parsed.
        """
        endpoint = CREATE_REASONING_STRATEGY_V2
        if automatic_publish:
            endpoint = f"{endpoint}?automaticPublish=true"

        headers = {
            "Authorization": self.api_service.token,
            "ProjectId": self.project_id,
            "Content-Type": "application/json",
            "Accept": "application/json"
        }
        data = {
            "strategyDefinition": {
                "name": name,
                "systemPrompt": system_prompt,
                "accessScope": access_scope,
                "type": strategy_type,
                "localizedDescriptions": localized_descriptions or []
            }
        }

        logger.debug(f"Creating reasoning strategy with data: {data}")

        response = self.api_service.post(endpoint=endpoint, headers=headers, data=data)
        validate_status_code(response)
        return parse_json_response(response, f"create reasoning strategy for project {self.project_id}")


    def update_reasoning_strategy(
            self,
            reasoning_strategy_id: str,
            name: str = None,
            system_prompt: str = None,
            access_scope: str = None,
            strategy_type: str = None,
            localized_descriptions: list = None,
            automatic_publish: bool = False,
            upsert: bool = False
    ):
        """
        Updates or upserts a reasoning strategy in the specified project.

        :param reasoning_strategy_id: str - Unique identifier of the strategy.
        :param name: str, optional - Updated strategy name.
        :param system_prompt: str, optional - Updated system prompt.
        :param access_scope: str, optional - Updated access scope, "public" or "private".
        :param strategy_type: str, optional - Updated strategy type, e.g., "addendum".
        :param localized_descriptions: list, optional - Updated localized descriptions.
        :param automatic_publish: bool, optional - Publish strategy after update (default: False).
        :param upsert: bool, optional - Create strategy if it doesn't exist (default: False).
        :return: dict - Updated strategy details.
        :raises ValueError: If access_scope is not "public" or "private", or strategy_type is not "addendum".
        :raises InvalidAPIResponseException: If the API response cannot be parsed.
        """
        if access_scope is not None:
            valid_access_scopes = ["public", "private"]
            if access_scope not in valid_access_scopes:
                raise ValueError("Access scope must be either 'public' or 'private'.")
        if strategy_type is not None:
            valid_types = ["addendum"]
            if strategy_type not in valid_types:
                raise ValueError("Type must be 'addendum'.")

        data = {
            "strategyDefinition": {}
        }
        if name is not None:
            data["strategyDefinition"]["name"] = name
        if system_prompt is not None:
            data["strategyDefinition"]["systemPrompt"] = system_prompt
        if access_scope is not None:
            data["strategyDefinition"]["accessScope"] = access_scope
        if strategy_type is not None:
            data["strategyDefinition"]["type"] = strategy_type
        if localized_descriptions is not None:
            data["strategyDefinition"]["localizedDescriptions"] = localized_descriptions

        logger.debug(f"Updating reasoning strategy with ID {reasoning_strategy_id} with data: {data}")

        endpoint = UPSERT_REASONING_STRATEGY_V2 if upsert else UPDATE_REASONING_STRATEGY_V2
        endpoint = endpoint.format(reasoningStrategyId=reasoning_strategy_id)

        if automatic_publish:
            endpoint = f"{endpoint}?automaticPublish=true"

        headers = {
            "Authorization": self.api_service.token,
            "ProjectId": self.project_id,
            "Content-Type": "application/json",
            "Accept": "application/json"
        }
        response = self.api_service.put(
            endpoint=endpoint,
            headers=headers,
            data=data
        )
        validate_status_code(response)
        return parse_json_response(response, f"update reasoning strategy {reasoning_strategy_id} in project {self.project_id}")


    def get_reasoning_strategy(
            self,
            reasoning_strategy_id: str = None,
            reasoning_strategy_name: str = None
    ):
        """
        Retrieves a reasoning strategy by ID or name in the specified project.

        :param reasoning_strategy_id: str, optional - Unique identifier of the strategy.
        :param reasoning_strategy_name: str, optional - Name of the strategy.
        :return: dict - Strategy details.
        :raises ValueError: If neither reasoning_strategy_id nor reasoning_strategy_name is provided.
        :raises InvalidAPIResponseException: If the API response cannot be parsed.
        """
        if not (reasoning_strategy_id or reasoning_strategy_name):
            raise ValueError("Either reasoning_strategy_id or reasoning_strategy_name must be provided.")

        identifier = reasoning_strategy_id if reasoning_strategy_id else reasoning_strategy_name
        endpoint = GET_REASONING_STRATEGY_V2.format(reasoningStrategyId=identifier)

        headers = {
            "Authorization": self.api_service.token,
            "ProjectId": self.project_id
        }

        if reasoning_strategy_id:
            logger.debug(f"Retrieving reasoning strategy detail with ID {reasoning_strategy_id}")
        else:
            logger.debug(f"Retrieving reasoning strategy detail with name {reasoning_strategy_name}")

        response = self.api_service.get(
            endpoint=endpoint,
            headers=headers
        )
        strategy_identifier = reasoning_strategy_id or reasoning_strategy_name
        validate_status_code(response)
        return parse_json_response(response, f"retrieve reasoning strategy {strategy_identifier} for project {self.project_id}")

    


from pygeai.core.files.managers import FileManager
from pygeai.core.files.models import File

file = File(id="7fe2393d-8f86-4020-a51f-bc14ab957e1e")

file_manager = FileManager()

response = file_manager.delete_file(file_id=file.id)
print(response)



"""
Agent Migration Examples

Demonstrates how to migrate agents between projects.
Uses project-scope API keys.
"""

from pygeai.migration.strategies import AgentMigrationStrategy


def example_migrate_specific_agents():
    """
    Migrate specific agents by ID to an existing project.
    Uses project-scope API keys.
    """
    agent_ids = [
        "9d8e7f6g-5h4i-3j2k-1l0m-9n8o7p6q5r4s",
        "1a2b3c4d-5e6f-7g8h-9i0j-1k2l3m4n5o6p"
    ]
    
    strategy = AgentMigrationStrategy(
        from_api_key="source_project_key_123abc456def789ghi012jkl345mno678pqr901",
        from_project_id="7x8y9z0a-1b2c-3d4e-5f6g-7h8i9j0k1l2m",
        from_instance="https://api.test.example.ai",
        to_api_key="dest_project_key_789stu012vwx345yz678abc901def234ghi567",
        to_project_id="3c4d5e6f-7g8h-9i0j-1k2l-3m4n5o6p7q8r",
        to_instance="https://api.prod.example.ai",
        agent_ids=agent_ids
    )
    
    result = strategy.migrate()
    
    print(f"Migrated {len(result['migrated_agents'])} agents")
    for agent in result['migrated_agents']:
        print(f"  - {agent['name']} (ID: {agent['id']})")
    
    return result


def example_migrate_all_agents():
    """
    Automatically discover and migrate all agents from source project.
    """
    strategy = AgentMigrationStrategy(
        from_api_key="source_project_key_123abc456def789ghi012jkl345mno678pqr901",
        from_project_id="7x8y9z0a-1b2c-3d4e-5f6g-7h8i9j0k1l2m",
        from_instance="https://api.test.example.ai",
        to_api_key="dest_project_key_789stu012vwx345yz678abc901def234ghi567",
        to_project_id="3c4d5e6f-7g8h-9i0j-1k2l-3m4n5o6p7q8r",
        to_instance="https://api.prod.example.ai",
        agent_ids=None  # None means "all agents"
    )
    
    result = strategy.migrate()
    print(f"Migrated all {len(result['migrated_agents'])} agents")
    
    return result


def example_custom_filtered_migration():
    """
    Create a custom migration that only migrates agents with specific tags
    or matching certain criteria.
    """
    from pygeai.lab.managers import AILabManager
    
    # Initialize managers for source and destination
    source_lab = AILabManager(
        api_key="source_project_key_123abc456def789ghi012jkl345mno678pqr901",
        project_id="7x8y9z0a-1b2c-3d4e-5f6g-7h8i9j0k1l2m",
        instance="https://api.test.example.ai"
    )
    
    # Get all agents and filter
    all_agents = source_lab.get_agent_list()
    
    # Custom filter: only migrate agents with "production" in their name
    filtered_agent_ids = [
        agent.id for agent in all_agents.agents 
        if "production" in agent.name.lower()
    ]
    
    print(f"Found {len(filtered_agent_ids)} agents matching filter")
    
    # Now migrate only the filtered agents
    strategy = AgentMigrationStrategy(
        from_api_key="source_project_key_123abc456def789ghi012jkl345mno678pqr901",
        from_project_id="7x8y9z0a-1b2c-3d4e-5f6g-7h8i9j0k1l2m",
        from_instance="https://api.test.example.ai",
        to_api_key="dest_project_key_789stu012vwx345yz678abc901def234ghi567",
        to_project_id="3c4d5e6f-7g8h-9i0j-1k2l-3m4n5o6p7q8r",
        agent_ids=filtered_agent_ids
    )
    
    result = strategy.migrate()
    print(f"Migrated {len(result['migrated_agents'])} filtered agents")
    
    return result


if __name__ == "__main__":
    print("Agent Migration Examples")
    print("=" * 80)
    print("\nThese examples demonstrate various agent migration patterns.")
    print("Uncomment to run specific examples:\n")
    
    # Uncomment to run specific examples:
    # example_migrate_specific_agents()
    # example_migrate_all_agents()
    # example_custom_filtered_migration()



from unittest import TestCase
import uuid
from pygeai.lab.managers import AILabManager
from pygeai.lab.models import Tool, ToolParameter
from pygeai.core.common.exceptions import APIResponseError


class TestAILabCreateToolIntegration(TestCase):    
    def setUp(self):
        """
        Set up the test environment.
        """
        self.ai_lab_manager = AILabManager()
        self.new_tool = self.__load_tool()
        self.created_tool: Tool = None


    def tearDown(self):
        """
        Clean up after each test if necessary.
        This can be used to delete the created tool
        """
        if isinstance(self.created_tool, Tool):

            self.ai_lab_manager.delete_tool(self.created_tool.id)

    
    def __load_tool(self):
        return Tool(
            name=str(uuid.uuid4()),
            description="Tool created for sdk testing purposes",
            scope="builtin",
            openApi="https://raw.usercontent.com//openapi.json",
            openApiJson={"openapi": "3.0.0","info": {"title": "Simple API overview","version": "2.0.0"}},
            accessScope="private",
            reportEvents="None",
            parameters=[{"key": "param", "description": "param description", "type":"app", "value":"param value", "data_type": "String", "isRequired": False}],
        )
    

    def __create_tool(self, tool=None, automatic_publish=False):
        """
        Helper to create a tool using ai_lab_manager.
        """
        return self.ai_lab_manager.create_tool(
            tool=self.new_tool if tool is None else tool,
            automatic_publish=automatic_publish
        )


    def test_create_tool_full_data(self):
        self.created_tool = self.__create_tool()
        created_tool = self.created_tool
        tool = self.new_tool
        self.assertTrue(isinstance(created_tool, Tool), "Expected a created tool")

        # Assert the main fields of the created tool
        self.assertIsNotNone(created_tool.id)
        self.assertEqual(created_tool.name, tool.name)
        self.assertEqual(created_tool.description, tool.description)
        self.assertEqual(created_tool.scope, tool.scope)
        self.assertEqual(created_tool.access_scope, tool.access_scope)
        self.assertEqual(created_tool.open_api, tool.open_api)
        self.assertEqual(created_tool.status, "active")

        # Assert agentData fields
        tool_param = created_tool.parameters[0]
        self.assertTrue(isinstance(tool_param, ToolParameter), "Expected parameters to be of type ToolParameter")
        self.assertEqual(tool_param.key, tool.parameters[0].key)
        self.assertEqual(tool_param.data_type, tool.parameters[0].data_type)
        self.assertEqual(tool_param.description, tool.parameters[0].description)
        self.assertEqual(tool_param.is_required, tool.parameters[0].is_required)
        self.assertEqual(tool_param.type, tool.parameters[0].type)
        self.assertEqual(tool_param.value, tool.parameters[0].value)        


    def test_create_tool_minimum_required_data(self):
        self.new_tool = Tool(
            name=str(uuid.uuid4()),
            description="Tool created for sdk testing purposes",
            scope="builtin"
        )
        self.created_tool = self.__create_tool()
        tool = self.new_tool

        self.assertIsNotNone(self.created_tool.id)
        self.assertEqual(self.created_tool.name, tool.name)
        self.assertEqual(self.created_tool.description, tool.description)


    def test_create_tool_without_required_data(self):
        test_params = [ True, False ]

        for auto_publish in test_params:
            
            with self.subTest(input=auto_publish): 
                self.new_tool = Tool(
                        name=str(uuid.uuid4())
                    )
                created_tool = self.__create_tool(automatic_publish=auto_publish)
                self.assertTrue(isinstance(created_tool, Tool), "Expected a created tool")    


    def test_create_tool_no_name(self):
        test_params = [ True, False ]

        for auto_publish in test_params:
            with self.subTest(input=auto_publish):
                self.new_tool.name = ""
                with self.assertRaises(APIResponseError) as exception:
                    self.__create_tool(automatic_publish=auto_publish)

                self.assertIn(
                    "Tool name cannot be empty.",
                    str(exception.exception),
                    f"Expected an error about the missing tool name with autopublish {'enabled' if auto_publish else 'disabled'}"
                )


    def test_create_tool_duplicated_name(self):
        test_params = [ True, False ]

        for auto_publish in test_params:

            with self.subTest(input=auto_publish):
                self.new_tool.name = "sdk_project_gemini_tool"
                with self.assertRaises(APIResponseError) as exception:
                    self.__create_tool(automatic_publish=auto_publish)
                self.assertIn(
                    "Tool already exists [name=sdk_project_gemini_tool]..",
                    str(exception.exception),                    
                    f"Expected an error about duplicated tool name with autopublish {'enabled' if auto_publish else 'disabled'}"
                )


    def test_create_tool_invalid_name(self):
        test_params = [ True, False ]

        for auto_publish in test_params:
            with self.subTest(input=auto_publish):
                new_tool = self.__load_tool()
                new_tool2 = self.__load_tool()

                with self.assertRaises(APIResponseError) as exception:
                    new_tool.name = f"{new_tool.name}:invalid"
                    self.__create_tool(tool=new_tool, automatic_publish=auto_publish)
                self.assertIn(
                    "Invalid character in name (: is not allowed).",
                    str(exception.exception),                    
                    f"Expected an error about invalid character (:) in tool name with autopublish {'enabled' if auto_publish else 'disabled'}"
                )

                with self.assertRaises(APIResponseError) as exception:
                    new_tool2.name = f"{new_tool2.name}/invalid"
                    self.__create_tool(tool=new_tool2, automatic_publish=auto_publish)
                self.assertIn(
                    "Invalid character in name (/ is not allowed).",
                    str(exception.exception),                    
                    f"Expected an error about invalid character (/) in tool name with autopublish {'enabled' if auto_publish else 'disabled'}"
                )
        

    def test_create_tool_invalid_access_scope(self):
        self.new_tool.access_scope = "project" 
        with self.assertRaises(ValueError) as exc:
            self.__create_tool()
        self.assertEqual(
            str(exc.exception),
            "Access scope must be one of public, private.",
            "Expected a ValueError exception for invalid access scope"
        )   


    def test_create_tool_default_scope(self):
        self.new_tool.access_scope = None
        self.created_tool = self.__create_tool()

        self.assertEqual(self.created_tool.access_scope, "private", "Expected the default access scope to be 'private' when not specified")


    def test_create_tool_no_public_name(self):
        test_params = [ True, False ]

        for auto_publish in test_params:

            with self.subTest(input=auto_publish):
                self.new_tool.access_scope = "public"
                self.new_tool.public_name = None
                with self.assertRaises(APIResponseError) as exception:
                    self.__create_tool(automatic_publish=auto_publish)
                self.assertIn(
                    "Tool publicName is required for tools with accessScope=public.",
                    str(exception.exception),                    
                    f"Expected an error about missing publicName for public access scope with autopublish {'enabled' if auto_publish else 'disabled'}"
                )

   
    def test_create_tool_invalid_public_name(self):
        test_params = [ True, False ]

        for auto_publish in test_params:
            with self.subTest(input=auto_publish): 
                self.new_tool.access_scope = "public"
                self.new_tool.public_name = "com.sdk.testing#"  # Add invalid character to public name
                with self.assertRaises(APIResponseError) as exception:
                    self.__create_tool(automatic_publish=auto_publish)

                self.assertIn(
                    "Invalid public name, it can only contain lowercase letters, numbers, periods (.), dashes (-), and underscores (_). Please remove any other characters.",
                    str(exception.exception),                    
                    f"The expected error about invalid publicName was not returned when autopublish is {'enabled' if auto_publish else 'disabled'}"
                )

    
    def test_create_tool_duplicated_public_name(self):
        test_params = [ True, False ]

        # Set the scope as public and assign a public name
        self.new_tool.access_scope = "public"
        self.new_tool.public_name=f"public_{self.new_tool.name}"
        self.created_tool = self.__create_tool()

        for auto_publish in test_params:
            with self.subTest(input=auto_publish):

                # Create a new with the same public name of created_tool
                duplicated_pn_tool = self.__load_tool()
                duplicated_pn_tool.access_scope = "public"
                duplicated_pn_tool.public_name = self.created_tool.public_name
                
                with self.assertRaises(APIResponseError) as exception:
                    self.__create_tool(tool=duplicated_pn_tool, automatic_publish=auto_publish)
                self.assertIn(
                    f"Tool already exists [publicName={self.created_tool.public_name}].",
                    str(exception.exception),                   
                    f"Expected an error about the duplicated public name when autopublish is {'enabled' if auto_publish else 'disabled'}"
                )

    
    def test_create_tool_api_scope_with_no_open_api(self):
        self.new_tool.scope = "api"
        with self.assertRaises(ValueError) as exception:
            self.new_tool.openApi = ""
        self.assertIn(
            '"Tool" object has no field "openApi"',
            str(exception.exception),
            "Expected a validation error when openApi is not provided for api scope"
        )

    
    def test_create_tool_api_scope_with_no_open_api_json(self):
        self.new_tool.scope = "api"
        with self.assertRaises(ValueError) as exception:
            self.new_tool.openApiJson = ""
            
        self.assertIn(
            '"Tool" object has no field "openApiJson"',
            str(exception.exception),
            "Expected a validation error when openApiJson is not provided for api scope"
        )

    
    def test_create_tool_invalid_scope(self):
        test_params = [ True, False ]

        for auto_publish in test_params:
            with self.subTest(input=auto_publish): 
                
                with self.assertRaises(ValueError) as exception:
                    self.new_tool.scope = "source"
                    self.__create_tool()
                self.assertIn(
                    'Scope must be one of builtin, external, api, proxied',
                    str(exception.exception),
                    "Expected a validation error about allowed values for instructions"
                )


    def test_create_tool_autopublish(self): 
        self.created_tool = self.__create_tool(automatic_publish=True)
        self.assertFalse(self.created_tool.is_draft, "Expected the tool to be published automatically")
    
    
    def test_create_tool_autopublish_private_scope(self):
        self.new_tool.access_scope = "private"

        self.created_tool = self.__create_tool(automatic_publish=True)
        self.assertFalse(self.created_tool.is_draft, "Expected the tool to be published automatically even with private scope")